@@ -55,11 +55,16 @@ powervm_opts = [
                default='localdisk',
                help='The disk driver to use for PowerVM disks. '
                'Valid options are: localdisk, ssp'),
-    cfg.StrOpt('pvm_vswitch_for_ovs',
-               default='OpenStackOVS',
+    cfg.StrOpt('volume_adapter',
+               choices=['fibre_channel', 'iscsi'], ignore_case=True,
+               default='fibre_channel',
+               help='The volume adapter to use for PowerVM volumes. '
+               'Valid options are: fibre_channel, iscsi'),
+    cfg.StrOpt('pvm_vswitch_for_novalink_io',
+               default='NovaLinkVEABridge',
                help="Name of the PowerVM virtual switch to be used when "
-                    "mapping Open vSwitch ports to PowerVM virtual Ethernet "
-                    "devices"),
+                    "mapping Linux based network ports to PowerVM virtual "
+                    "Ethernet devices"),
     cfg.BoolOpt('use_rmc_mgmt_vif',
                 default=True,
                 help="If enabled, tells the PowerVM Driver to create an RMC "
@@ -119,6 +124,12 @@ vol_adapter_opts = [
                     'specified and at least one fabric_X_port_wwpns option '
                     '(where X corresponds to the fabric name) must be '
                     'specified.'),
+    cfg.StrOpt('network_attach_strategy',
+               choices=['iscsi'], ignore_case=True,
+               default='iscsi',
+               help='The iSCSI Volume Strategy defines how iSCSI Cinder '
+                    'volumes should be attached to the Virtual Machine. The '
+                    'option is: iscsi.'),
     cfg.StrOpt('fc_npiv_adapter_api',
                default='nova_powervm.virt.powervm.volume.npiv.'
                'NPIVVolumeAdapter',
@@ -126,7 +137,7 @@ vol_adapter_opts = [
                     'connection mechanism.'),
     cfg.StrOpt('fc_vscsi_adapter_api',
                default='nova_powervm.virt.powervm.volume.vscsi.'
-               'VscsiVolumeAdapter',
+               'PVVscsiFCVolumeAdapter',
                help='Volume Adapter API to connect FC volumes through Virtual '
                     'I/O Server using PowerVM vSCSI connection mechanism.'),
     cfg.IntOpt('vscsi_vios_connections_required',
@@ -187,8 +198,20 @@ swift_opts = [
                'file.  Example: /etc/swiftclient/myca.pem')
 ]
 
+vnc_opts = [
+    cfg.BoolOpt('vnc_use_x509_auth', default=False,
+                help='If enabled, uses X509 Authentication for the '
+                'VNC sessions started for each VM.'),
+    cfg.StrOpt('vnc_ca_certs', help='Path to CA certificate '
+               'to use for verifying VNC X509 Authentication.'),
+    cfg.StrOpt('vnc_server_cert', help='Path to Server certificate '
+               'to use for verifying VNC X509 Authentication.'),
+    cfg.StrOpt('vnc_server_key', help='Path to Server private key '
+               'to use for verifying VNC X509 Authentication.')
+]
+
 STATIC_OPTIONS = (powervm_opts + localdisk_opts + ssp_opts + vol_adapter_opts
-                  + npiv_opts + remote_restart_opts + swift_opts)
+                  + npiv_opts + remote_restart_opts + swift_opts + vnc_opts)
 
 # Dictionary where the key is the NPIV Fabric Name, and the value is a list of
 # Physical WWPNs that match the key.
