@@ -14,6 +14,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import fixtures
 import mock
 
 import copy
@@ -60,6 +61,11 @@ class TestLocalDisk(test.TestCase):
         vg_uuid = 'd5065c2c-ac43-3fa6-af32-ea84a3960291'
         self.mock_vg_uuid.return_value = ('vios_uuid', vg_uuid)
 
+        # Return the mgmt uuid
+        self.mgmt_uuid = self.useFixture(fixtures.MockPatch(
+            'nova_powervm.virt.powervm.mgmt.mgmt_uuid')).mock
+        self.mgmt_uuid.return_value = 'mp_uuid'
+
     def tearDown(self):
         test.TestCase.tearDown(self)
 
@@ -68,8 +74,7 @@ class TestLocalDisk(test.TestCase):
 
     @staticmethod
     def get_ls(adpt):
-        return ld.LocalStorage({'adapter': adpt, 'host_uuid': 'host_uuid',
-                                'mp_uuid': 'mp_uuid'})
+        return ld.LocalStorage(adpt, 'host_uuid')
 
     @mock.patch('pypowervm.tasks.storage.upload_new_vdisk')
     @mock.patch('nova_powervm.virt.powervm.disk.driver.'
@@ -109,7 +114,7 @@ class TestLocalDisk(test.TestCase):
         self.assertEqual(3072.0, local.capacity_used)
 
     @mock.patch('pypowervm.tasks.scsi_mapper.remove_maps')
-    @mock.patch('nova_powervm.virt.powervm.vios.get_active_vioses')
+    @mock.patch('pypowervm.tasks.partition.get_active_vioses')
     def test_disconnect_image_disk(self, mock_active_vioses, mock_rm_maps):
         # vio_to_vg is a single-entry response.  Wrap it and put it in a list
         # to act as the feed for FeedTaskFx and FeedTask.
@@ -138,7 +143,7 @@ class TestLocalDisk(test.TestCase):
                                              match_func=mock.ANY)
 
     @mock.patch('pypowervm.tasks.scsi_mapper.remove_maps')
-    @mock.patch('nova_powervm.virt.powervm.vios.get_active_vioses')
+    @mock.patch('pypowervm.tasks.partition.get_active_vioses')
     def test_disconnect_image_disk_no_update(self, mock_active_vioses,
                                              mock_rm_maps):
         # vio_to_vg is a single-entry response.  Wrap it and put it in a list
@@ -192,7 +197,7 @@ class TestLocalDisk(test.TestCase):
 
     @mock.patch('pypowervm.tasks.scsi_mapper.build_vscsi_mapping')
     @mock.patch('pypowervm.tasks.scsi_mapper.add_map')
-    @mock.patch('nova_powervm.virt.powervm.vios.get_active_vioses')
+    @mock.patch('pypowervm.tasks.partition.get_active_vioses')
     def test_connect_image_disk(self, mock_active_vioses, mock_add_map,
                                 mock_build_map):
         # vio_to_vg is a single-entry response.  Wrap it and put it in a list
@@ -221,7 +226,7 @@ class TestLocalDisk(test.TestCase):
 
     @mock.patch('pypowervm.tasks.scsi_mapper.build_vscsi_mapping')
     @mock.patch('pypowervm.tasks.scsi_mapper.add_map')
-    @mock.patch('nova_powervm.virt.powervm.vios.get_active_vioses')
+    @mock.patch('pypowervm.tasks.partition.get_active_vioses')
     def test_connect_image_disk_no_update(self, mock_active_vioses,
                                           mock_add_map, mock_build_map):
         # vio_to_vg is a single-entry response.  Wrap it and put it in a list
@@ -400,6 +405,11 @@ class TestLocalDiskFindVG(test.TestCase):
         self.mock_vios_feed = [pvm_vios.VIOS.wrap(self.vio_to_vg)]
         self.mock_vg_feed = [pvm_stor.VG.wrap(self.vg_to_vio)]
 
+        # Return the mgmt uuid
+        self.mgmt_uuid = self.useFixture(fixtures.MockPatch(
+            'nova_powervm.virt.powervm.mgmt.mgmt_uuid')).mock
+        self.mgmt_uuid.return_value = 'mp_uuid'
+
     @mock.patch('pypowervm.wrappers.storage.VG.wrap')
     @mock.patch('pypowervm.wrappers.virtual_io_server.VIOS.wrap')
     def test_get_vg_uuid(self, mock_vio_wrap, mock_vg_wrap):
@@ -411,9 +421,7 @@ class TestLocalDiskFindVG(test.TestCase):
         mock_vg_wrap.return_value = self.mock_vg_feed
         self.flags(volume_group_name='rootvg', group='powervm')
 
-        storage = ld.LocalStorage({'adapter': self.apt,
-                                   'host_uuid': 'host_uuid',
-                                   'mp_uuid': 'mp_uuid'})
+        storage = ld.LocalStorage(self.apt, 'host_uuid')
 
         # Make sure the uuids match
         self.assertEqual('d5065c2c-ac43-3fa6-af32-ea84a3960291',
@@ -435,5 +443,4 @@ class TestLocalDiskFindVG(test.TestCase):
                    volume_group_vios_name='invalid_vios', group='powervm')
 
         self.assertRaises(npvmex.VGNotFound, ld.LocalStorage,
-                          {'adapter': self.apt, 'host_uuid': 'host_uuid',
-                           'mp_uuid': 'mp_uuid'})
+                          self.apt, 'host_uuid')
