@@ -21,6 +21,7 @@ import fixtures
 import logging
 import mock
 from oslo_serialization import jsonutils
+import six
 
 from nova import block_device as nova_block_device
 from nova.compute import task_states
@@ -43,12 +44,14 @@ from pypowervm.wrappers import base_partition as pvm_bp
 from pypowervm.wrappers import logical_partition as pvm_lpar
 from pypowervm.wrappers import virtual_io_server as pvm_vios
 
+from nova.virt.powervm import driver
+
 from nova_powervm.tests.virt import powervm
 from nova_powervm.tests.virt.powervm import fixtures as fx
-from nova_powervm.virt.powervm import driver
 from nova_powervm.virt.powervm import exception as p_exc
 from nova_powervm.virt.powervm import live_migration as lpm
 from nova_powervm.virt.powervm import vm
+from nova_powervm.virt.powervm import volume as vol_attach
 
 LOG = logging.getLogger(__name__)
 logging.basicConfig()
@@ -100,8 +103,12 @@ class TestPowerVMDriver(test.TestCase):
         self._setup_lpm()
 
         self.disk_dvr = self.drv.disk_dvr
-        self.vol_fix = self.useFixture(fx.VolumeAdapter())
+        self.vol_fix = self.useFixture(fx.VolumeAdapter(
+            'nova_powervm.virt.powervm.volume.vscsi.PVVscsiFCVolumeAdapter'))
         self.vol_drv = self.vol_fix.drv
+        self.iscsi_vol_fix = self.useFixture(fx.VolumeAdapter(
+            'nova_powervm.virt.powervm.volume.iscsi.IscsiVolumeAdapter'))
+        self.iscsi_vol_drv = self.iscsi_vol_fix.drv
 
         self.crt_lpar = self.useFixture(fixtures.MockPatch(
             'nova_powervm.virt.powervm.vm.crt_lpar')).mock
@@ -110,7 +117,7 @@ class TestPowerVMDriver(test.TestCase):
             'nova_powervm.virt.powervm.vm.get_instance_wrapper')).mock
 
         self.build_tx_feed = self.useFixture(fixtures.MockPatch(
-            'nova_powervm.virt.powervm.vios.build_tx_feed_task')).mock
+            'pypowervm.tasks.partition.build_active_vio_feed_task')).mock
 
         self.stg_ftsk = pvm_tx.FeedTask('fake', pvm_vios.VIOS.getter(self.apt))
         self.build_tx_feed.return_value = self.stg_ftsk
@@ -122,11 +129,28 @@ class TestPowerVMDriver(test.TestCase):
             'pypowervm.util.sanitize_partition_name_for_api')).mock
         self.san_lpar_name.side_effect = lambda name: name
 
+        self.validate_vopt = self.useFixture(fixtures.MockPatch(
+            'pypowervm.tasks.vopt.validate_vopt_repo_exists')).mock
+        self.validate_vopt.return_value = None, None
+
+        self.slot_mgr = self.useFixture(fixtures.MockPatch(
+            'nova_powervm.virt.powervm.slot.build_slot_mgr')).mock.return_value
+
         # Create an instance to test with
         self.inst = objects.Instance(**powervm.TEST_INST_SPAWNING)
         self.inst_ibmi = objects.Instance(**powervm.TEST_INST_SPAWNING)
         self.inst_ibmi.system_metadata = {'image_os_distro': 'ibmi'}
 
+    def test_driver_capabilities(self):
+        """Test the default capabilities."""
+        test_driver = driver.PowerVMDriver(fake.FakeVirtAPI())
+        self.assertFalse(test_driver.capabilities['has_imagecache'])
+        self.assertTrue(test_driver.capabilities['supports_recreate'])
+        self.assertFalse(
+            test_driver.capabilities['supports_migrate_to_same_host'])
+        self.assertTrue(test_driver.capabilities['supports_attach_interface'])
+        self.assertFalse(test_driver.capabilities['supports_device_tagging'])
+
     def _setup_lpm(self):
         """Setup the lpm environment.
 
@@ -138,6 +162,15 @@ class TestPowerVMDriver(test.TestCase):
         self.lpm_inst.uuid = 'inst1'
         self.drv.live_migrations = {'inst1': self.lpm}
 
+    def _vol_drv_maps(self):
+        VOLUME_DRIVER_MAPPINGS = {
+            'fibre_channel': vol_attach.FC_STRATEGY_MAPPING[
+                driver.CONF.powervm.fc_attach_strategy.lower()],
+            'iscsi': vol_attach.NETWORK_STRATEGY_MAPPING[
+                driver.CONF.powervm.network_attach_strategy.lower()],
+        }
+        return VOLUME_DRIVER_MAPPINGS
+
     def test_driver_create(self):
         """Validates that a driver of the PowerVM type can be initialized."""
         test_drv = driver.PowerVMDriver(fake.FakeVirtAPI())
@@ -150,15 +183,21 @@ class TestPowerVMDriver(test.TestCase):
 
     def test_get_volume_connector(self):
         """Tests that a volume connector can be built."""
+
+        self.flags(volume_adapter='fibre_channel', group='powervm')
         vol_connector = self.drv.get_volume_connector(mock.Mock())
         self.assertIsNotNone(vol_connector['wwpns'])
         self.assertIsNotNone(vol_connector['host'])
 
-    def test_get_disk_adapter(self):
+        self.flags(volume_adapter='iscsi', group='powervm')
+        vol_connector = self.drv.get_volume_connector(mock.Mock())
+        self.assertIsNotNone(vol_connector['initiator'])
+
+    def test_setup_disk_adapter(self):
         # Ensure we can handle upper case option and we instantiate the class
         self.flags(disk_driver='LoCaLDisK', group='powervm')
         self.drv.disk_dvr = None
-        self.drv._get_disk_adapter()
+        self.drv._setup_disk_adapter()
         # The local disk driver has been mocked, so we just compare the name
         self.assertIn('LocalStorage()', str(self.drv.disk_dvr))
 
@@ -261,7 +300,7 @@ class TestPowerVMDriver(test.TestCase):
                 '.execute')
     @mock.patch('nova_powervm.virt.powervm.tasks.storage.CreateDiskForImg'
                 '.execute')
-    @mock.patch('nova_powervm.virt.powervm.driver.PowerVMDriver.'
+    @mock.patch('nova.virt.powervm.driver.PowerVMDriver.'
                 '_is_booted_from_volume')
     @mock.patch('nova_powervm.virt.powervm.tasks.network.PlugMgmtVif.execute')
     @mock.patch('nova_powervm.virt.powervm.tasks.network.PlugVifs.execute')
@@ -295,7 +334,7 @@ class TestPowerVMDriver(test.TestCase):
         self.assertTrue(mock_crt_disk_img.called)
         self.crt_lpar.assert_called_with(
             self.apt, self.drv.host_wrapper, self.inst, self.inst.get_flavor(),
-            nvram=None)
+            nvram=None, slot_mgr=self.slot_mgr)
         self.assertTrue(mock_pwron.called)
         self.assertFalse(mock_pwron.call_args[1]['synchronous'])
         # Assert that tasks that are not supposed to be called are not called
@@ -308,14 +347,11 @@ class TestPowerVMDriver(test.TestCase):
     @mock.patch('nova_powervm.virt.powervm.tasks.network.PlugVifs.execute')
     @mock.patch('nova_powervm.virt.powervm.media.ConfigDrivePowerVM.'
                 'create_cfg_drv_vopt')
-    @mock.patch('nova_powervm.virt.powervm.media.ConfigDrivePowerVM.'
-                '_validate_vopt_vg')
     @mock.patch('nova.virt.configdrive.required_by')
     @mock.patch('nova.objects.flavor.Flavor.get_by_id')
     @mock.patch('pypowervm.tasks.power.power_on')
     def test_spawn_with_cfg(self, mock_pwron, mock_get_flv, mock_cfg_drv,
-                            mock_val_vopt, mock_cfg_vopt, mock_plug_vifs,
-                            mock_plug_mgmt_vif):
+                            mock_cfg_vopt, mock_plug_vifs, mock_plug_mgmt_vif):
         """Validates the PowerVM spawn w/ config drive operations."""
         # Set up the mocks to the tasks.
         mock_get_flv.return_value = self.inst.get_flavor()
@@ -328,10 +364,10 @@ class TestPowerVMDriver(test.TestCase):
         # Create LPAR was called
         self.crt_lpar.assert_called_with(self.apt, self.drv.host_wrapper,
                                          self.inst, self.inst.get_flavor(),
-                                         nvram=None)
+                                         nvram=None, slot_mgr=self.slot_mgr)
         # Config drive was called
-        self.assertTrue(mock_val_vopt.called)
         self.assertTrue(mock_cfg_vopt.called)
+        self.assertTrue(self.validate_vopt.called)
 
         # Power on was called
         self.assertTrue(mock_pwron.called)
@@ -342,7 +378,7 @@ class TestPowerVMDriver(test.TestCase):
     @mock.patch('nova.virt.block_device.DriverVolumeBlockDevice.save')
     @mock.patch('nova_powervm.virt.powervm.tasks.storage.CreateDiskForImg'
                 '.execute')
-    @mock.patch('nova_powervm.virt.powervm.driver.PowerVMDriver.'
+    @mock.patch('nova.virt.powervm.driver.PowerVMDriver.'
                 '_is_booted_from_volume')
     @mock.patch('nova_powervm.virt.powervm.tasks.network.PlugMgmtVif.execute')
     @mock.patch('nova_powervm.virt.powervm.tasks.network.PlugVifs.execute')
@@ -380,7 +416,7 @@ class TestPowerVMDriver(test.TestCase):
         # Create LPAR was called
         self.crt_lpar.assert_called_with(self.apt, self.drv.host_wrapper,
                                          self.inst, self.inst.get_flavor(),
-                                         nvram=None)
+                                         nvram=None, slot_mgr=self.slot_mgr)
         # Power on was called
         self.assertTrue(mock_pwron.called)
         self.assertFalse(mock_pwron.call_args[1]['synchronous'])
@@ -397,7 +433,7 @@ class TestPowerVMDriver(test.TestCase):
     @mock.patch('nova.virt.block_device.DriverVolumeBlockDevice.save')
     @mock.patch('nova_powervm.virt.powervm.tasks.storage.CreateDiskForImg'
                 '.execute')
-    @mock.patch('nova_powervm.virt.powervm.driver.PowerVMDriver.'
+    @mock.patch('nova.virt.powervm.driver.PowerVMDriver.'
                 '_is_booted_from_volume')
     @mock.patch('nova_powervm.virt.powervm.tasks.network.PlugMgmtVif.execute')
     @mock.patch('nova_powervm.virt.powervm.tasks.network.PlugVifs.execute')
@@ -444,7 +480,7 @@ class TestPowerVMDriver(test.TestCase):
         # Create LPAR was called
         self.crt_lpar.assert_called_with(self.apt, self.drv.host_wrapper,
                                          self.inst, self.inst.get_flavor(),
-                                         nvram=None)
+                                         nvram=None, slot_mgr=self.slot_mgr)
         # Power on was called
         self.assertTrue(mock_pwron.called)
         self.assertFalse(mock_pwron.call_args[1]['synchronous'])
@@ -458,7 +494,7 @@ class TestPowerVMDriver(test.TestCase):
     @mock.patch('nova.virt.block_device.DriverVolumeBlockDevice.save')
     @mock.patch('nova_powervm.virt.powervm.tasks.storage.CreateDiskForImg'
                 '.execute')
-    @mock.patch('nova_powervm.virt.powervm.driver.PowerVMDriver.'
+    @mock.patch('nova.virt.powervm.driver.PowerVMDriver.'
                 '_is_booted_from_volume')
     @mock.patch('nova_powervm.virt.powervm.tasks.network.PlugMgmtVif.execute')
     @mock.patch('nova_powervm.virt.powervm.tasks.network.PlugVifs.execute')
@@ -493,7 +529,7 @@ class TestPowerVMDriver(test.TestCase):
         # Create LPAR was called
         self.crt_lpar.assert_called_with(self.apt, self.drv.host_wrapper,
                                          self.inst, self.inst.get_flavor(),
-                                         nvram=None)
+                                         nvram=None, slot_mgr=self.slot_mgr)
         # Power on was called
         self.assertTrue(mock_pwron.called)
         self.assertFalse(mock_pwron.call_args[1]['synchronous'])
@@ -512,7 +548,7 @@ class TestPowerVMDriver(test.TestCase):
     @mock.patch('nova_powervm.virt.powervm.tasks.storage.ConnectVolume')
     @mock.patch('nova_powervm.virt.powervm.tasks.storage.ConnectDisk')
     @mock.patch('nova_powervm.virt.powervm.tasks.storage.FindDisk')
-    @mock.patch('nova_powervm.virt.powervm.driver.PowerVMDriver.'
+    @mock.patch('nova.virt.powervm.driver.PowerVMDriver.'
                 '_is_booted_from_volume')
     @mock.patch('nova_powervm.virt.powervm.tasks.network.PlugMgmtVif')
     @mock.patch('nova_powervm.virt.powervm.tasks.network.PlugVifs')
@@ -545,12 +581,14 @@ class TestPowerVMDriver(test.TestCase):
         # Invoke the method.
         self.drv.spawn('context', self.inst, powervm.EMPTY_IMAGE,
                        'injected_files', 'admin_password')
-
-        # Recreate uses all XAGs.
-        self.build_tx_feed.assert_called_once_with(
-            self.drv.adapter, self.drv.host_uuid, xag={pvm_const.XAG.VIO_FMAP,
-                                                       pvm_const.XAG.VIO_STOR,
-                                                       pvm_const.XAG.VIO_SMAP})
+        xags = {pvm_const.XAG.VIO_FMAP, pvm_const.XAG.VIO_SMAP,
+                pvm_const.XAG.VIO_STOR}
+        calls = [mock.call(self.drv.adapter, xag=xags),
+                 mock.call(self.drv.adapter, name='create_scrubber',
+                           xag=xags - {pvm_const.XAG.VIO_STOR})]
+        # Recreate uses all XAGs, builds special FeedTask for immediate
+        # scrubbing.
+        self.build_tx_feed.assert_has_calls(calls)
         # _vol_drv_iter gets called once in spawn itself, and once under
         # _add_volume_connection_tasks.
         # TODO(IBM): Find a way to make the call just once.  Unless it's cheap.
@@ -571,7 +609,7 @@ class TestPowerVMDriver(test.TestCase):
         self.assertTrue(mock_find_disk.called)
         self.crt_lpar.assert_called_with(
             self.apt, self.drv.host_wrapper, self.inst, self.inst.get_flavor(),
-            nvram='nvram data')
+            nvram='nvram data', slot_mgr=mock_build_slot_mgr.return_value)
         # SaveSlotStore.execute
         mock_build_slot_mgr.return_value.save.assert_called_once_with()
         self.assertTrue(mock_pwron.called)
@@ -609,7 +647,7 @@ class TestPowerVMDriver(test.TestCase):
         # Create LPAR was called
         self.crt_lpar.assert_called_with(self.apt, self.drv.host_wrapper,
                                          self.inst, self.inst.get_flavor(),
-                                         nvram=None)
+                                         nvram=None, slot_mgr=self.slot_mgr)
         self.assertEqual(2, self.vol_drv.connect_volume.call_count)
 
         # Power on was called
@@ -620,17 +658,17 @@ class TestPowerVMDriver(test.TestCase):
         self.assertEqual(2, self.vol_drv.disconnect_volume.call_count)
 
     @mock.patch('nova.virt.block_device.DriverVolumeBlockDevice.save')
-    @mock.patch('nova_powervm.virt.powervm.tasks.storage.CreateDiskForImg'
-                '.execute')
-    @mock.patch('nova_powervm.virt.powervm.driver.PowerVMDriver.'
+    @mock.patch('nova_powervm.virt.powervm.tasks.storage.CreateDiskForImg.'
+                'execute')
+    @mock.patch('nova.virt.powervm.driver.PowerVMDriver.'
                 '_is_booted_from_volume')
     @mock.patch('nova_powervm.virt.powervm.tasks.network.PlugMgmtVif.execute')
     @mock.patch('nova_powervm.virt.powervm.tasks.network.PlugVifs.execute')
     @mock.patch('nova.virt.configdrive.required_by')
     @mock.patch('nova.objects.flavor.Flavor.get_by_id')
-    @mock.patch('nova_powervm.virt.powervm.tasks.vm.UpdateIBMiSettings'
-                '.execute')
-    @mock.patch('nova_powervm.virt.powervm.driver.PowerVMDriver.'
+    @mock.patch('nova_powervm.virt.powervm.tasks.vm.UpdateIBMiSettings.'
+                'execute')
+    @mock.patch('nova.virt.powervm.driver.PowerVMDriver.'
                 '_get_boot_connectivity_type')
     @mock.patch('pypowervm.tasks.power.power_on')
     def test_spawn_ibmi(self, mock_pwron, mock_boot_conn_type,
@@ -659,7 +697,7 @@ class TestPowerVMDriver(test.TestCase):
         self.crt_lpar.assert_called_with(self.apt, self.drv.host_wrapper,
                                          self.inst_ibmi,
                                          self.inst_ibmi.get_flavor(),
-                                         nvram=None)
+                                         nvram=None, slot_mgr=self.slot_mgr)
 
         self.assertTrue(mock_boot_conn_type.called)
         self.assertTrue(mock_update_lod_src.called)
@@ -680,7 +718,7 @@ class TestPowerVMDriver(test.TestCase):
                 '.execute')
     @mock.patch('nova_powervm.virt.powervm.tasks.storage.CreateDiskForImg'
                 '.execute')
-    @mock.patch('nova_powervm.virt.powervm.driver.PowerVMDriver.'
+    @mock.patch('nova.virt.powervm.driver.PowerVMDriver.'
                 '_is_booted_from_volume')
     @mock.patch('nova_powervm.virt.powervm.tasks.network.PlugMgmtVif.execute')
     @mock.patch('nova_powervm.virt.powervm.tasks.network.PlugVifs.execute')
@@ -688,7 +726,7 @@ class TestPowerVMDriver(test.TestCase):
     @mock.patch('nova.objects.flavor.Flavor.get_by_id')
     @mock.patch('nova_powervm.virt.powervm.tasks.vm.UpdateIBMiSettings'
                 '.execute')
-    @mock.patch('nova_powervm.virt.powervm.driver.PowerVMDriver.'
+    @mock.patch('nova.virt.powervm.driver.PowerVMDriver.'
                 '_get_boot_connectivity_type')
     @mock.patch('pypowervm.tasks.power.power_on')
     def test_spawn_ibmi_without_bdms(self, mock_pwron, mock_boot_conn_type,
@@ -717,7 +755,7 @@ class TestPowerVMDriver(test.TestCase):
         self.assertTrue(mock_crt_disk_img.called)
         self.crt_lpar.assert_called_with(
             self.apt, self.drv.host_wrapper, self.inst_ibmi,
-            self.inst_ibmi.get_flavor(), nvram=None)
+            self.inst_ibmi.get_flavor(), nvram=None, slot_mgr=self.slot_mgr)
         self.assertTrue(mock_update_lod_src.called)
         self.assertTrue(mock_pwron.called)
         self.assertFalse(mock_pwron.call_args[1]['synchronous'])
@@ -754,7 +792,7 @@ class TestPowerVMDriver(test.TestCase):
         # Create LPAR was called
         self.crt_lpar.assert_called_with(self.apt, self.drv.host_wrapper,
                                          self.inst, self.inst.get_flavor(),
-                                         nvram=None)
+                                         nvram=None, slot_mgr=self.slot_mgr)
 
         # Since the create disks method failed, the delete disks should not
         # have been called
@@ -792,7 +830,7 @@ class TestPowerVMDriver(test.TestCase):
         # Create LPAR was called
         self.crt_lpar.assert_called_with(self.apt, self.drv.host_wrapper,
                                          self.inst, self.inst.get_flavor(),
-                                         nvram=None)
+                                         nvram=None, slot_mgr=self.slot_mgr)
         self.assertEqual(1, self.vol_drv.connect_volume.call_count)
 
         # Power on should not be called.  Shouldn't get that far in flow.
@@ -822,7 +860,13 @@ class TestPowerVMDriver(test.TestCase):
         ret = self.drv._is_booted_from_volume(None)
         self.assertFalse(ret)
 
-    def test_get_inst_xag(self):
+    @mock.patch('nova_powervm.virt.powervm.driver.VOLUME_DRIVER_MAPPINGS')
+    def test_get_inst_xag(self, mock_mapping):
+        def getitem(name):
+            return self._vol_drv_maps()[name]
+        mock_mapping.__getitem__.side_effect = getitem
+
+        self.flags(volume_adapter='fibre_channel', group='powervm')
         # No volumes - should be just the SCSI mapping
         xag = self.drv._get_inst_xag(mock.Mock(), None)
         self.assertEqual([pvm_const.XAG.VIO_SMAP], xag)
@@ -834,6 +878,7 @@ class TestPowerVMDriver(test.TestCase):
 
         # The NPIV volume attach - requires SCSI, Storage and FC Mapping
         self.flags(fc_attach_strategy='npiv', group='powervm')
+        mock_mapping.return_value = self._vol_drv_maps()
         xag = self.drv._get_inst_xag(mock.Mock(), [mock.Mock()])
         self.assertEqual({pvm_const.XAG.VIO_STOR,
                           pvm_const.XAG.VIO_SMAP,
@@ -841,6 +886,13 @@ class TestPowerVMDriver(test.TestCase):
 
         # The vSCSI Volume attach - Ensure case insensitive.
         self.flags(fc_attach_strategy='VSCSI', group='powervm')
+        mock_mapping.return_value = self._vol_drv_maps()
+        xag = self.drv._get_inst_xag(mock.Mock(), [mock.Mock()])
+        self.assertEqual([pvm_const.XAG.VIO_SMAP], xag)
+
+        # The iSCSI volume attach - only nees the SCSI mapping.
+        self.flags(volume_adapter='iscsi', group='powervm')
+        mock_mapping.return_value = self._vol_drv_maps()
         xag = self.drv._get_inst_xag(mock.Mock(), [mock.Mock()])
         self.assertEqual([pvm_const.XAG.VIO_SMAP], xag)
 
@@ -889,29 +941,37 @@ class TestPowerVMDriver(test.TestCase):
         flow.add.assert_has_calls([mock.call('disconn_vol1'),
                                    mock.call('disconn_vol2')])
 
+    @mock.patch('pypowervm.tasks.scsi_mapper.remove_maps')
+    @mock.patch('pypowervm.wrappers.entry_wrapper.EntryWrapper.update',
+                new=mock.Mock())
     @mock.patch('nova_powervm.virt.powervm.tasks.network.UnplugVifs.execute')
-    @mock.patch('nova_powervm.virt.powervm.driver.PowerVMDriver.'
+    @mock.patch('nova.virt.powervm.driver.PowerVMDriver.'
                 '_is_booted_from_volume')
     @mock.patch('nova_powervm.virt.powervm.vm.dlt_lpar')
     @mock.patch('nova_powervm.virt.powervm.vm.power_off')
     @mock.patch('nova_powervm.virt.powervm.media.ConfigDrivePowerVM.'
                 'dlt_vopt')
-    @mock.patch('nova_powervm.virt.powervm.media.ConfigDrivePowerVM.'
-                '_validate_vopt_vg')
     @mock.patch('nova_powervm.virt.powervm.vm.get_pvm_uuid')
+    @mock.patch('nova.virt.configdrive.required_by')
     @mock.patch('nova.objects.flavor.Flavor.get_by_id')
     @mock.patch('nova_powervm.virt.powervm.slot.build_slot_mgr')
-    def test_destroy_internal(self, mock_bld_slot_mgr, mock_get_flv,
-                              mock_pvmuuid, mock_val_vopt, mock_dlt_vopt,
-                              mock_pwroff, mock_dlt, mock_boot_from_vol,
-                              mock_unplug_vifs):
+    def test_destroy_internal(
+            self, mock_bld_slot_mgr, mock_get_flv, mock_cfg, mock_pvmuuid,
+            mock_dlt_vopt, mock_pwroff, mock_dlt, mock_boot_from_vol,
+            mock_unplug_vifs, mock_rm_maps):
         """Validates the basic PowerVM destroy."""
         # NVRAM Manager
         self.drv.nvram_mgr = mock.Mock()
+        mock_cfg.return_value = True
 
         # BDMs
         mock_bdms = self._fake_bdms()
         mock_boot_from_vol.return_value = False
+
+        def validate_rm_maps(vwrap, lpar_uuid):
+            self.assertIsInstance(vwrap, pvm_vios.VIOS)
+            self.assertEqual(mock_pvmuuid.return_value, lpar_uuid)
+        mock_rm_maps.side_effect = validate_rm_maps
         # Invoke the method.
         self.drv.destroy('context', self.inst, ['net'],
                          block_device_info=mock_bdms)
@@ -933,6 +993,10 @@ class TestPowerVMDriver(test.TestCase):
         # Validate that the volume detach was called
         self.vol_drv.disconnect_volume.assert_has_calls(
             [mock.call(mock_bld_slot_mgr.return_value)] * 2)
+
+        # Post-scrub was invoked
+        mock_rm_maps.assert_called()
+
         # Delete LPAR was called
         mock_dlt.assert_called_with(self.apt, mock.ANY)
 
@@ -1044,23 +1108,20 @@ class TestPowerVMDriver(test.TestCase):
         assert_not_called()
 
     @mock.patch('nova_powervm.virt.powervm.tasks.network.UnplugVifs.execute')
-    @mock.patch('nova_powervm.virt.powervm.driver.PowerVMDriver.'
+    @mock.patch('nova.virt.powervm.driver.PowerVMDriver.'
                 '_is_booted_from_volume')
     @mock.patch('nova_powervm.virt.powervm.vm.dlt_lpar')
     @mock.patch('nova_powervm.virt.powervm.vm.power_off')
     @mock.patch('nova_powervm.virt.powervm.media.ConfigDrivePowerVM.'
                 'dlt_vopt')
-    @mock.patch('nova_powervm.virt.powervm.media.ConfigDrivePowerVM.'
-                '_validate_vopt_vg')
     @mock.patch('nova_powervm.virt.powervm.vm.get_pvm_uuid')
+    @mock.patch('nova.virt.configdrive.required_by')
     @mock.patch('nova.objects.flavor.Flavor.get_by_id')
     @mock.patch('nova_powervm.virt.powervm.slot.build_slot_mgr')
-    def test_destroy_internal_no_nvram_cleanup(self, mock_bld_slot_mgr,
-                                               mock_get_flv, mock_pvmuuid,
-                                               mock_val_vopt, mock_dlt_vopt,
-                                               mock_pwroff, mock_dlt,
-                                               mock_boot_from_vol,
-                                               mock_unplug_vifs):
+    def test_destroy_internal_no_nvram_cleanup(
+            self, mock_bld_slot_mgr, mock_get_flv, mock_cfg, mock_pvmuuid,
+            mock_dlt_vopt, mock_pwroff, mock_dlt, mock_boot_from_vol,
+            mock_unplug_vifs):
         """Validates the basic PowerVM destroy, without NVRAM cleanup.
 
         Used to validate the behavior when destroying evacuated instances.
@@ -1069,6 +1130,7 @@ class TestPowerVMDriver(test.TestCase):
         # NVRAM Manager
         self.drv.nvram_mgr = mock.Mock()
         self.inst.host = 'other'
+        mock_cfg.return_value = True
 
         # BDMs
         mock_bdms = self._fake_bdms()
@@ -1211,15 +1273,14 @@ class TestPowerVMDriver(test.TestCase):
     @mock.patch('nova_powervm.virt.powervm.vm.power_off')
     @mock.patch('nova_powervm.virt.powervm.media.ConfigDrivePowerVM.'
                 'dlt_vopt')
-    @mock.patch('nova_powervm.virt.powervm.media.ConfigDrivePowerVM.'
-                '_validate_vopt_vg')
+    @mock.patch('nova.virt.configdrive.required_by')
     @mock.patch('nova.objects.flavor.Flavor.get_by_id')
-    def test_destroy_rollback(self, mock_get_flv, mock_val_vopt,
-                              mock_dlt_vopt, mock_pwroff, mock_dlt,
-                              mock_unplug_vifs):
+    def test_destroy_rollback(self, mock_get_flv, mock_cfg, mock_dlt_vopt,
+                              mock_pwroff, mock_dlt, mock_unplug_vifs):
         """Validates the basic PowerVM destroy rollback mechanism works."""
         # Set up the mocks to the tasks.
         mock_get_flv.return_value = self.inst.get_flavor()
+        mock_cfg.return_value = True
 
         # BDMs
         mock_bdms = self._fake_bdms()
@@ -1273,13 +1334,13 @@ class TestPowerVMDriver(test.TestCase):
         boot_flav = objects.Flavor(vcpus=1, memory_mb=2048, root_gb=12)
         # Tasks expected to be added for migrate
         expected = [
-            'pwr_off_lpar',
+            'pwr_off_vm',
             'store_nvram',
             'extend_disk_boot',
             'disconnect_vol_*',
             'disconnect_vol_*',
             'fake',
-            'rename_lpar_migrate_instance-00000001',
+            'rename_vm_migrate_instance-00000001',
         ]
         dest_host = host + '1'
         with fx.DriverTaskFlow() as taskflow_fix:
@@ -1320,7 +1381,7 @@ class TestPowerVMDriver(test.TestCase):
 
         # Tasks expected to be added for migration to different host
         expected = [
-            'crt_lpar',
+            'crt_vm',
             'plug_vifs',
             'plug_mgmt_vif',
             'find_disk',
@@ -1330,8 +1391,8 @@ class TestPowerVMDriver(test.TestCase):
             'connect_vol_*',
             'save_bdm_fake_vol2',
             'fake',
-            'get_lpar',
-            'pwr_lpar',
+            'get_vm',
+            'pwr_vm',
         ]
         with fx.DriverTaskFlow() as taskflow_fix:
             self.drv.finish_migration(
@@ -1341,9 +1402,9 @@ class TestPowerVMDriver(test.TestCase):
                 self.inst, self.drv.store_api, adapter=self.drv.adapter,
                 vol_drv_iter=mock.ANY)
             taskflow_fix.assert_tasks_added(self, expected)
-            # Slot manager was passed to PlugVifs, PlugMgmtVif, and
+            # Slot manager was passed to Create, PlugVifs, PlugMgmtVif, and
             # connect_volume (twice)
-            for idx in (1, 2, 5, 7):
+            for idx in (0, 1, 2, 5, 7):
                 self.assertEqual(mock_bld_slot_mgr.return_value,
                                  taskflow_fix.tasks_added[idx].slot_mgr)
         self.san_lpar_name.assert_not_called()
@@ -1352,14 +1413,14 @@ class TestPowerVMDriver(test.TestCase):
 
         # Tasks expected to be added for resize to the same host
         expected = [
-            'resize_lpar',
+            'resize_vm',
             'connect_vol_*',
             'save_bdm_fake_vol1',
             'connect_vol_*',
             'save_bdm_fake_vol2',
             'fake',
-            'get_lpar',
-            'pwr_lpar',
+            'get_vm',
+            'pwr_vm',
         ]
         with fx.DriverTaskFlow() as taskflow_fix:
             self.drv.finish_migration(
@@ -1381,7 +1442,7 @@ class TestPowerVMDriver(test.TestCase):
         # Tasks expected to be added for resize to the same host, no BDMS,
         # and no power_on
         expected = [
-            'resize_lpar',
+            'resize_vm',
         ]
         with fx.DriverTaskFlow() as taskflow_fix:
             self.drv.finish_migration(
@@ -1447,7 +1508,7 @@ class TestPowerVMDriver(test.TestCase):
         self.assertTrue(mock_task_pwr.power_on.called)
         self.assertFalse(mock_task_pwr.power_on.call_args[1]['synchronous'])
 
-    @mock.patch('nova_powervm.virt.powervm.driver.vm')
+    @mock.patch('nova_powervm.virt.powervm.vm')
     @mock.patch('nova_powervm.virt.powervm.tasks.vm.vm')
     @mock.patch('nova_powervm.virt.powervm.tasks.vm.power')
     def test_unrescue(self, mock_task_pwr, mock_task_vm, mock_dvr_vm):
@@ -1462,7 +1523,7 @@ class TestPowerVMDriver(test.TestCase):
         self.assertTrue(mock_task_pwr.power_on.called)
         self.assertFalse(mock_task_pwr.power_on.call_args[1]['synchronous'])
 
-    @mock.patch.object(driver, 'LOG')
+    @mock.patch('nova_powervm.virt.powervm.driver.LOG')
     def test_log_op(self, mock_log):
         """Validates the log_operations."""
         self.drv._log_operation('fake_op', self.inst)
@@ -1475,13 +1536,13 @@ class TestPowerVMDriver(test.TestCase):
                     'op': 'fake_op'}
         mock_log.info.assert_called_with(entry, msg_dict)
 
-    def test_host_resources(self):
-        # Mock methods not currently under test
-        with mock.patch.object(self.apt, 'read') as mock_read:
-            mock_read.return_value = None
-            # Run the actual test
-            stats = self.drv.get_available_resource('nodename')
-            self.assertIsNotNone(stats)
+    @mock.patch('pypowervm.wrappers.managed_system.System.get',
+                new=mock.Mock(return_value=[mock.Mock()]))
+    @mock.patch('nova_powervm.virt.powervm.host.build_host_resource_from_ms')
+    def test_host_resources(self, mock_bhrfm):
+        mock_bhrfm.return_value = {}
+        stats = self.drv.get_available_resource('nodename')
+        self.assertIsNotNone(stats)
 
         # Check for the presence of fields added to host stats
         fields = ('local_gb', 'local_gb_used')
@@ -1513,8 +1574,8 @@ class TestPowerVMDriver(test.TestCase):
 
         # Mock up the network info.  They get sanitized to upper case.
         net_info = [
-            {'address': 'aa:bb:cc:dd:ee:ff'},
-            {'address': 'aa:bb:cc:dd:ee:22'}
+            {'address': 'aa:bb:cc:dd:ee:ff', 'vnic_type': 'normal'},
+            {'address': 'aa:bb:cc:dd:ee:22', 'vnic_type': 'normal'}
         ]
 
         # Mock up the rmc vswitch
@@ -1530,9 +1591,9 @@ class TestPowerVMDriver(test.TestCase):
 
         # The create should have only been called once.  The other was already
         # existing.
-        mock_plug_vif.assert_called_once_with(
+        mock_plug_vif.assert_called_with(
             self.drv.adapter, self.drv.host_uuid, self.inst, net_info[1],
-            mock_bld_slot_mgr.return_value)
+            mock_bld_slot_mgr.return_value, new_vif=True)
         mock_bld_slot_mgr.return_value.save.assert_called_once_with()
         self.assertEqual(0, mock_plug_rmc_vif.call_count)
 
@@ -1624,7 +1685,7 @@ class TestPowerVMDriver(test.TestCase):
     def test_get_host_ip_addr(self):
         self.assertEqual(self.drv.get_host_ip_addr(), '127.0.0.1')
 
-    @mock.patch('nova_powervm.virt.powervm.driver.LOG.warning')
+    @mock.patch('nova.virt.powervm.driver.LOG.warning')
     @mock.patch('nova.compute.utils.get_machine_ips')
     def test_get_host_ip_addr_failure(self, mock_ips, mock_log):
         mock_ips.return_value = ['1.1.1.1']
@@ -1713,7 +1774,39 @@ class TestPowerVMDriver(test.TestCase):
         self.assertEqual('uuid', resp.internal_access_path)
 
         mock_vterm.assert_called_once_with(
-            mock.ANY, 'uuid', mock.ANY, vnc_path='uuid')
+            mock.ANY, 'uuid', mock.ANY, vnc_path='uuid', use_x509_auth=False,
+            ca_certs=None, server_cert=None, server_key=None)
+
+    @mock.patch('pypowervm.tasks.vterm.open_remotable_vnc_vterm')
+    @mock.patch('nova_powervm.virt.powervm.vm.get_pvm_uuid')
+    def test_get_vnc_console_failure(self, mock_uuid, mock_vterm):
+        # Mock response
+        mock_vterm.side_effect = pvm_exc.VNCBasedTerminalFailedToOpen(
+            err='test')
+        mock_uuid.return_value = 'uuid'
+
+        # Invoke
+        self.assertRaises(exc.ConsoleTypeUnavailable, self.drv.get_vnc_console,
+                          mock.ANY, self.inst)
+
+    @mock.patch('pypowervm.tasks.vterm.open_remotable_vnc_vterm')
+    @mock.patch('nova_powervm.virt.powervm.vm.get_pvm_uuid')
+    def test_get_vnc_console_notfound(self, mock_uuid, mock_vterm):
+        # Mock response
+        mock_resp = mock.Mock(status=404)
+        mock_vterm.side_effect = pvm_exc.HttpError(mock_resp)
+        mock_uuid.return_value = 'uuid'
+        exc_str = None
+
+        # Invoke
+        try:
+            self.drv.get_vnc_console(mock.ANY, self.inst)
+        except Exception as exc:
+            exc_str = six.text_type(exc)
+
+        self.assertIsNotNone(exc_str, 'Expected Not Found Exception')
+        self.assertEqual(exc_str, 'Unable to open console since '
+                         'virtual machine Fake Instance does not exist.')
 
     @staticmethod
     def _fake_bdms():
@@ -1763,7 +1856,7 @@ class TestPowerVMDriver(test.TestCase):
         self.assertEqual('dest_data', dest_data)
 
     def test_can_live_mig_dest_clnup(self):
-        self.drv.check_can_live_migrate_destination_cleanup(
+        self.drv.cleanup_live_migration_destination_check(
             'context', 'dest_data')
 
     @mock.patch('nova_powervm.virt.powervm.live_migration.LiveMigrationSrc')
