@@ -14,26 +14,18 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from __future__ import absolute_import
+
+import fixtures
 import mock
 
 from nova import test
 from pypowervm.tests import test_fixtures as pvm_fx
-from pypowervm.tests.test_utils import pvmhttp
-from pypowervm.wrappers import storage as pvm_stor
+from pypowervm.wrappers import storage as pvm_stg
 from pypowervm.wrappers import virtual_io_server as pvm_vios
 
-from nova_powervm.virt.powervm import exception as npvmex
 from nova_powervm.virt.powervm import media as m
 
-VOL_GRP_DATA = 'fake_volume_group2.txt'
-VOL_GRP_NOVG_DATA = 'fake_volume_group_no_vg.txt'
-
-VOL_GRP_WITH_VIOS = 'fake_volume_group_with_vio_data.txt'
-VIOS_WITH_VOL_GRP = 'fake_vios_with_volume_group_data.txt'
-
-VIOS_NO_VG = 'fake_vios_feed_no_vg.txt'
-VIOS_FEED = 'fake_vios_feed2.txt'
-
 
 class TestConfigDrivePowerVM(test.TestCase):
     """Unit Tests for the ConfigDrivePowerVM class."""
@@ -43,29 +35,13 @@ class TestConfigDrivePowerVM(test.TestCase):
 
         self.apt = self.useFixture(pvm_fx.AdapterFx()).adpt
 
-        def resp(file_name):
-            return pvmhttp.load_pvm_resp(
-                file_name, adapter=self.apt).get_response()
+        self.validate_vopt = self.useFixture(fixtures.MockPatch(
+            'pypowervm.tasks.vopt.validate_vopt_repo_exists')).mock
+        self.validate_vopt.return_value = None, None
 
-        self.vol_grp_resp = resp(VOL_GRP_DATA)
-        self.vol_grp_novg_resp = resp(VOL_GRP_NOVG_DATA)
-
-        self.vg_to_vio = resp(VOL_GRP_WITH_VIOS)
-        self.vio_to_vg = resp(VIOS_WITH_VOL_GRP)
-
-        self.vio_feed_no_vg = resp(VIOS_NO_VG)
-        self.vio_feed = resp(VIOS_FEED)
-
-        # Wipe out the static variables, so that the revalidate is called
-        m.ConfigDrivePowerVM._cur_vios_uuid = None
-        m.ConfigDrivePowerVM._cur_vios_name = None
-        m.ConfigDrivePowerVM._cur_vg_uuid = None
-
-    @mock.patch('nova_powervm.virt.powervm.media.ConfigDrivePowerVM.'
-                '_validate_vopt_vg')
     @mock.patch('nova.api.metadata.base.InstanceMetadata')
     @mock.patch('nova.virt.configdrive.ConfigDriveBuilder.make_drive')
-    def test_crt_cfg_dr_iso(self, mock_mkdrv, mock_meta, mock_vopt_valid):
+    def test_crt_cfg_dr_iso(self, mock_mkdrv, mock_meta):
         """Validates that the image creation method works."""
         cfg_dr_builder = m.ConfigDrivePowerVM(self.apt, 'host_uuid')
         mock_instance = mock.MagicMock()
@@ -86,19 +62,18 @@ class TestConfigDrivePowerVM(test.TestCase):
         self.assertEqual('cfg_fake_instance_with_name_that_.iso', file_name)
         self.assertEqual('/tmp/cfgdrv/cfg_fake_instance_with_name_that_.iso',
                          iso_path)
+        self.assertTrue(self.validate_vopt.called)
 
     @mock.patch('nova_powervm.virt.powervm.media.ConfigDrivePowerVM.'
                 '_create_cfg_dr_iso')
-    @mock.patch('nova_powervm.virt.powervm.media.ConfigDrivePowerVM.'
-                '_validate_vopt_vg')
     @mock.patch('os.path.getsize')
     @mock.patch('os.remove')
     @mock.patch('nova_powervm.virt.powervm.media.ConfigDrivePowerVM.'
                 '_upload_vopt')
     @mock.patch('nova_powervm.virt.powervm.media.ConfigDrivePowerVM.'
                 '_attach_vopt')
-    def test_crt_cfg_drv_vopt(self, mock_attach, mock_upld,
-                              mock_rm, mock_size, mock_validate, mock_cfg_iso):
+    def test_crt_cfg_drv_vopt(self, mock_attach, mock_upld, mock_rm, mock_size,
+                              mock_cfg_iso):
         # Mock Returns
         mock_cfg_iso.return_value = '/tmp/cfgdrv/fake.iso', 'fake.iso'
         mock_size.return_value = 10000
@@ -111,23 +86,27 @@ class TestConfigDrivePowerVM(test.TestCase):
         self.assertTrue(mock_upld.called)
         self.assertTrue(mock_attach.called)
         mock_attach.assert_called_with(mock.ANY, 'fake_lpar', mock.ANY, None)
+        self.assertTrue(self.validate_vopt.called)
 
     @mock.patch('pypowervm.tasks.scsi_mapper.add_map')
     @mock.patch('pypowervm.tasks.scsi_mapper.build_vscsi_mapping')
-    @mock.patch('nova_powervm.virt.powervm.media.ConfigDrivePowerVM.'
-                '_validate_vopt_vg')
-    def test_attach_vopt(self, mock_validate, mock_build_map, mock_add_map):
-        # to act as the feed for FeedTaskFx and FeedTask.
-        feed = [pvm_vios.VIOS.wrap(self.vio_to_vg)]
-        ft_fx = pvm_fx.FeedTaskFx(feed)
-        self.useFixture(ft_fx)
-
+    @mock.patch('pypowervm.utils.transaction.WrapperTask')
+    def test_attach_vopt(self, mock_class_wrapper_task, mock_build_map,
+                         mock_add_map):
+        # Create objects to test with
         mock_instance = mock.MagicMock(name='fake-instance')
-
         cfg_dr_builder = m.ConfigDrivePowerVM(self.apt, 'fake_host')
-        cfg_dr_builder.vios_uuid = feed[0].uuid
         vopt = mock.Mock()
-        self.apt.read.return_value = self.vio_to_vg
+        mock_vios = mock.Mock(spec=pvm_vios.VIOS)
+        mock_vios.configure_mock(name='vios name')
+
+        # Mock methods not currently under test
+        mock_wrapper_task = mock.MagicMock()
+        mock_class_wrapper_task.return_value = mock_wrapper_task
+
+        def call_param(param):
+            param(mock_vios)
+        mock_wrapper_task.add_functor_subtask.side_effect = call_param
 
         def validate_build(host_uuid, vios_w, lpar_uuid, vopt_elem):
             self.assertEqual('fake_host', host_uuid)
@@ -143,16 +122,27 @@ class TestConfigDrivePowerVM(test.TestCase):
             return 'added'
         mock_add_map.side_effect = validate_add
 
+        # Run the actual test
         cfg_dr_builder._attach_vopt(mock_instance, 'lpar_uuid', vopt)
 
         # Make sure they were called and validated
+        self.assertTrue(mock_wrapper_task.execute.called)
         self.assertEqual(1, mock_build_map.call_count)
         self.assertEqual(1, mock_add_map.call_count)
-        self.assertEqual(1, ft_fx.patchers['update'].mock.call_count)
+        self.assertTrue(self.validate_vopt.called)
 
-    @mock.patch('nova_powervm.virt.powervm.media.ConfigDrivePowerVM.'
-                '_validate_vopt_vg')
-    def test_mgmt_cna_to_vif(self, mock_validate):
+    def test_sanitize_network_info(self):
+        network_info = [{'type': 'lbr'}, {'type': 'pvm_sea'},
+                        {'type': 'ovs'}]
+
+        cfg_dr_builder = m.ConfigDrivePowerVM(self.apt, 'fake_host')
+
+        resp = cfg_dr_builder._sanitize_network_info(network_info)
+        expected_ret = [{'type': 'vif'}, {'type': 'vif'},
+                        {'type': 'ovs'}]
+        self.assertEqual(resp, expected_ret)
+
+    def test_mgmt_cna_to_vif(self):
         mock_cna = mock.MagicMock()
         mock_cna.mac = "FAD4433ED120"
 
@@ -184,93 +174,48 @@ class TestConfigDrivePowerVM(test.TestCase):
         self.assertEqual('fe80::fdff:ffff:feff:ffff',
                          m.ConfigDrivePowerVM._mac_to_link_local(mac))
 
-    def test_validate_opt_vg(self):
-        self.apt.read.side_effect = [self.vio_feed, self.vol_grp_resp]
-        vg_update = self.vol_grp_resp.feed.entries[0]
-        self.apt.update_by_path.return_value = vg_update
-        cfg_dr_builder = m.ConfigDrivePowerVM(self.apt, 'fake_host')
-        self.assertEqual('1e46bbfd-73b6-3c2a-aeab-a1d3f065e92f',
-                         cfg_dr_builder.vg_uuid)
-
-    def test_validate_opt_vg_fail(self):
-        self.apt.read.side_effect = [self.vio_feed_no_vg,
-                                     self.vol_grp_novg_resp]
-        self.assertRaises(npvmex.NoMediaRepoVolumeGroupFound,
-                          m.ConfigDrivePowerVM, self.apt, 'fake_host')
-
-    @mock.patch('pypowervm.tasks.storage.rm_vg_storage')
-    @mock.patch('nova_powervm.virt.powervm.vm.get_vm_id')
     @mock.patch('nova_powervm.virt.powervm.media.ConfigDrivePowerVM.'
-                '_validate_vopt_vg')
-    def test_dlt_vopt(self, mock_vop_valid, mock_vm_id, rm_vg_stor):
-        feed = [pvm_vios.VIOS.wrap(self.vio_to_vg)]
-        ft_fx = pvm_fx.FeedTaskFx(feed)
-        self.useFixture(ft_fx)
-
-        # Set up the mock data.
-        self.apt.read.side_effect = [self.vio_to_vg, self.vg_to_vio]
-        mock_vm_id.return_value = '2'
-
-        # Make sure that the first update is a VIO and doesn't have the vopt
-        # mapping
-        def validate_update(*kargs, **kwargs):
-            vol_grp = kargs[0]
-            # This is the VG update.  Make sure there are no optical medias
-            # anymore.
-            self.assertEqual(0, len(vol_grp.vmedia_repos[0].optical_media))
-            return vol_grp.entry
-
-        self.apt.update_by_path.side_effect = validate_update
-
-        def validate_remove_stor(vg_w, vopts=[]):
-            self.assertIsInstance(vg_w, pvm_stor.VG)
-            self.assertEqual(1, len(vopts))
-            self.assertIsInstance(vopts[0], pvm_stor.VOptMedia)
-        rm_vg_stor.side_effect = validate_remove_stor
-
-        # Count the number of SCSI mappings beforehand
-        num_maps_before = len(feed[0].scsi_mappings)
+                'add_dlt_vopt_tasks')
+    @mock.patch('pypowervm.wrappers.virtual_io_server.VIOS.wrap',
+                new=mock.MagicMock())
+    @mock.patch('pypowervm.tasks.scsi_mapper.find_maps')
+    @mock.patch('pypowervm.utils.transaction.FeedTask')
+    @mock.patch('pypowervm.utils.transaction.FeedTask.execute')
+    def test_dlt_vopt_no_map(self, mock_execute, mock_class_feed_task,
+                             mock_add_dlt_vopt_tasks, mock_find_maps):
+        # Init objects to test with
+        mock_feed_task = mock.MagicMock()
+        mock_class_feed_task.return_value = mock_feed_task
+        mock_find_maps.return_value = []
 
         # Invoke the operation
         cfg_dr = m.ConfigDrivePowerVM(self.apt, 'fake_host')
-        cfg_dr.vios_uuid = feed[0].uuid
-        cfg_dr.dlt_vopt('2')
-
-        self.assertEqual(num_maps_before - 1, len(feed[0].scsi_mappings))
-        self.assertTrue(ft_fx.patchers['update'].mock.called)
-        self.assertTrue(rm_vg_stor.called)
-
-    @mock.patch('pypowervm.tasks.storage.rm_vg_storage')
-    @mock.patch('nova_powervm.virt.powervm.vm.get_vm_id')
-    @mock.patch('nova_powervm.virt.powervm.media.ConfigDrivePowerVM.'
-                '_validate_vopt_vg')
-    def test_dlt_vopt_no_map(self, mock_vop_valid, mock_vm_id, rm_vg_stor):
-        feed = [pvm_vios.VIOS.wrap(self.vio_to_vg)]
-        ft_fx = pvm_fx.FeedTaskFx(feed)
-        self.useFixture(ft_fx)
-
-        # Set up the mock data.
-        self.apt.read.side_effect = [self.vio_to_vg, self.vg_to_vio]
-        mock_vm_id.return_value = '2'
-
-        def validate_remove_stor(vg_w, vopts=[]):
-            self.assertIsInstance(vg_w, pvm_stor.VG)
-            self.assertEqual(1, len(vopts))
-            self.assertIsInstance(vopts[0], pvm_stor.VOptMedia)
-        rm_vg_stor.side_effect = validate_remove_stor
-
-        # Count the number of SCSI mappings beforehand
-        num_maps_before = len(feed[0].scsi_mappings)
-
-        # Invoke the operation
-        cfg_dr = m.ConfigDrivePowerVM(self.apt, 'fake_host')
-        cfg_dr.vios_uuid = feed[0].uuid
         cfg_dr.dlt_vopt('2', remove_mappings=False)
 
-        # The storage should have been removed
-        self.assertTrue(rm_vg_stor.called)
+        # Verify expected methods were called
+        mock_add_dlt_vopt_tasks.assert_not_called()
+        self.assertTrue(mock_feed_task.execute.called)
 
-        # The mappings should have changed
-        self.assertTrue(ft_fx.patchers['update'].mock.called)
-        # But the number should be the same (the vopt mapping was replaced)
-        self.assertEqual(num_maps_before, len(feed[0].scsi_mappings))
+    @mock.patch('nova_powervm.virt.powervm.vm.get_vm_id',
+                new=mock.MagicMock(return_value='2'))
+    @mock.patch('pypowervm.tasks.scsi_mapper.gen_match_func')
+    @mock.patch('pypowervm.tasks.scsi_mapper.find_maps')
+    def test_add_dlt_vopt_tasks(self, mock_find_maps, mock_gen_match_func):
+        # Init objects to test with
+        cfg_dr = m.ConfigDrivePowerVM(self.apt, 'fake_host')
+        stg_ftsk = mock.MagicMock()
+        cfg_dr.vios_uuid = 'vios_uuid'
+        lpar_uuid = 'lpar_uuid'
+        mock_find_maps.return_value = [mock.Mock(backing_storage='stor')]
+
+        # Run
+        cfg_dr.add_dlt_vopt_tasks(lpar_uuid, stg_ftsk)
+
+        # Validate
+        mock_gen_match_func.assert_called_with(pvm_stg.VOptMedia)
+        mock_find_maps.assert_called_with(
+            stg_ftsk.get_wrapper().scsi_mappings, client_lpar_id='2',
+            match_func=mock_gen_match_func.return_value)
+        self.assertTrue(stg_ftsk.add_post_execute.called)
+        self.assertTrue(
+            stg_ftsk.wrapper_tasks['vios_uuid'].add_functor_subtask.called)
