@@ -1,4 +1,4 @@
-# Copyright 2015 IBM Corp.
+# Copyright 2015, 2016 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -15,18 +15,22 @@
 #    under the License.
 
 import mock
-from oslo_config import cfg
 
 from nova.compute import task_states
+from oslo_serialization import jsonutils
+from pypowervm import const as pvm_const
+from pypowervm.tests.tasks import util as tju
 from pypowervm.tests import test_fixtures as pvm_fx
 from pypowervm.tests.test_utils import pvmhttp
 from pypowervm.wrappers import virtual_io_server as pvm_vios
 
+from nova_powervm import conf as cfg
 from nova_powervm.tests.virt.powervm.volume import test_driver as test_vol
 from nova_powervm.virt.powervm import exception as exc
 from nova_powervm.virt.powervm.volume import npiv
 
 VIOS_FEED = 'fake_vios_feed2.txt'
+VIOS_FEED_2 = 'fake_vios_feed.txt'
 
 CONF = cfg.CONF
 
@@ -46,6 +50,7 @@ class TestNPIVAdapter(test_vol.TestVolumeAdapter):
         self.wwpn1 = '21000024FF649104'
         self.wwpn2 = '21000024FF649107'
         self.vios_uuid = '3443DB77-AED1-47ED-9AA5-3DB9C6CF7089'
+        self.slot_mgr = mock.Mock()
 
         # Set up the transaction manager
         feed = pvm_vios.VIOS.wrap(self.vios_feed_resp)
@@ -94,6 +99,8 @@ class TestNPIVAdapter(test_vol.TestVolumeAdapter):
     def test_connect_volume(self, mock_add_map):
         # Mock
         self._basic_system_metadata(npiv.FS_UNMAPPED)
+        self.slot_mgr.build_map.get_vfc_slots = mock.Mock(
+            return_value=['62'])
 
         def add_map(vios_w, host_uuid, vm_uuid, port_map, **kwargs):
             self.assertIsInstance(vios_w, pvm_vios.VIOS)
@@ -104,33 +111,40 @@ class TestNPIVAdapter(test_vol.TestVolumeAdapter):
         mock_add_map.side_effect = add_map
 
         # Test connect volume
-        self.vol_drv.connect_volume()
+        self.vol_drv.connect_volume(self.slot_mgr)
 
         # Verify that the appropriate connections were made.
         self.assertEqual(1, mock_add_map.call_count)
+        mock_add_map.assert_called_once_with(
+            mock.ANY, 'host_uuid', '1234', ('21000024FF649104', 'AA BB'),
+            lpar_slot_num='62', provided={})
         self.assertEqual(1, self.ft_fx.patchers['update'].mock.call_count)
         self.assertEqual(npiv.FS_INST_MAPPED,
                          self.vol_drv._get_fabric_state('A'))
 
+        # Verify the correct post execute methods were added to the feed task
+        self.assertEqual('fab_slot_A_id',
+                         self.vol_drv.stg_ftsk._post_exec[0].name)
+        self.assertEqual('fab_A_id',
+                         self.vol_drv.stg_ftsk._post_exec[1].name)
+
     def test_connect_volume_not_valid(self):
         """Validates that a connect will fail if in a bad state."""
         self.mock_inst_wrap.can_modify_io.return_value = False, 'Invalid I/O'
-        self.assertRaises(exc.VolumeAttachFailed, self.vol_drv.connect_volume)
-
-    def test_connect_volume_bad_wwpn(self):
-        """Ensures an error is raised if a bad WWPN is used."""
-        self._basic_system_metadata(npiv.FS_UNMAPPED, p_wwpn='bad')
-        self.assertRaises(exc.VolumeAttachFailed, self.vol_drv.connect_volume)
+        self.assertRaises(exc.VolumeAttachFailed, self.vol_drv.connect_volume,
+                          self.slot_mgr)
 
     @mock.patch('pypowervm.tasks.vfc_mapper.add_map')
     def test_connect_volume_inst_mapped(self, mock_add_map):
         """Test if already connected to an instance, don't do anything"""
         self._basic_system_metadata(npiv.FS_INST_MAPPED)
         mock_add_map.return_value = None
+        self.slot_mgr.build_map.get_vfc_slots = mock.Mock(
+            return_value=['62'])
 
         # Test subsequent connect volume calls when the fabric is mapped with
         # inst partition
-        self.vol_drv.connect_volume()
+        self.vol_drv.connect_volume(self.slot_mgr)
 
         # Verify
         self.assertEqual(1, mock_add_map.call_count)
@@ -140,6 +154,38 @@ class TestNPIVAdapter(test_vol.TestVolumeAdapter):
         self.assertEqual(npiv.FS_INST_MAPPED,
                          self.vol_drv._get_fabric_state('A'))
 
+    @mock.patch('pypowervm.tasks.vfc_mapper.find_vios_for_wwpn')
+    @mock.patch('pypowervm.tasks.vfc_mapper.derive_npiv_map')
+    @mock.patch('nova_powervm.virt.powervm.volume.npiv.NPIVVolumeAdapter.'
+                '_set_fabric_meta')
+    def test_ensure_phys_ports(self, set_fabric_meta, derive_npiv_map,
+                               find_vios_for_wwpn):
+        """Test that an npiv mapping gets rebuilt for stale wwpns"""
+
+        self._basic_system_metadata(npiv.FS_INST_MAPPED, "stale-wwpn")
+        port_maps = [('stale-wwpn', 'AA BB')]
+        vios_wraps = [mock.MagicMock()]
+        fab = 'A'
+
+        # First the 'good' case where output should equal input
+        port_maps = [(self.wwpn1, 'AA BB')]
+        find_vios_for_wwpn.return_value = (vios_wraps[0], mock.MagicMock())
+        maps = self.vol_drv._ensure_phys_ports_for_system(port_maps,
+                                                          vios_wraps, fab)
+        self.assertEqual(port_maps, maps)
+        find_vios_for_wwpn.assert_called_once()
+        derive_npiv_map.assert_not_called()
+
+        # Now test the stale case - rebuild
+        port_maps = [('stale-wwpn', 'AA BB')]
+        expected_map = [(self.wwpn1, port_maps[0][1])]
+        derive_npiv_map.return_value = expected_map
+        find_vios_for_wwpn.return_value = (None, None)
+        maps = self.vol_drv._ensure_phys_ports_for_system(port_maps,
+                                                          vios_wraps, fab)
+        self.assertEqual(expected_map, maps)
+        derive_npiv_map.assert_called_once()
+
     def _basic_system_metadata(self, fabric_state, p_wwpn='21000024FF649104'):
         meta_fb_key = self.vol_drv._sys_meta_fabric_key('A')
         meta_fb_map = '%s,AA,BB' % p_wwpn
@@ -159,7 +205,7 @@ class TestNPIVAdapter(test_vol.TestVolumeAdapter):
         mock_find_vios.return_value = (mock.Mock(uuid=self.vios_uuid),)
 
         # Invoke
-        self.vol_drv.disconnect_volume()
+        self.vol_drv.disconnect_volume(self.slot_mgr)
 
         # Two maps removed on one VIOS
         self.assertEqual(2, mock_remove_maps.call_count)
@@ -176,7 +222,7 @@ class TestNPIVAdapter(test_vol.TestVolumeAdapter):
         mock_get_fabric_meta.return_value = []
 
         # Invoke
-        self.vol_drv.disconnect_volume()
+        self.vol_drv.disconnect_volume(self.slot_mgr)
 
         # No mappings should have been removed
         self.assertFalse(mock_remove_maps.called)
@@ -185,14 +231,37 @@ class TestNPIVAdapter(test_vol.TestVolumeAdapter):
         """Validates that a disconnect will fail if in a bad state."""
         self.mock_inst_wrap.can_modify_io.return_value = False, 'Bleh'
         self.assertRaises(exc.VolumeDetachFailed,
-                          self.vol_drv.disconnect_volume)
+                          self.vol_drv.disconnect_volume, self.slot_mgr)
+
+    @mock.patch('nova_powervm.virt.powervm.volume.npiv.NPIVVolumeAdapter.'
+                '_fabric_names')
+    def test_disconnect_volume_not_on_same_host(self, mock_names):
+        """Validates a disconnect still removes mapping when host's differ."""
+        self.vol_drv.instance.task_state = None
+        self.vol_drv.instance.host = 'not_host_in_conf'
+
+        self.vol_drv.disconnect_volume(self.slot_mgr)
+        mock_names.assert_called_once()
+
+    @mock.patch('nova_powervm.virt.powervm.volume.npiv.NPIVVolumeAdapter.'
+                '_fabric_names')
+    def test_disconnect_volume_spawning_not_on_same_host(self, mock_names):
+        """Validates disconnect when instance is spawning on another host."""
+        self.vol_drv.instance.task_state = 'spawning'
+        self.vol_drv.instance.host = 'not_host_in_conf'
+
+        self.vol_drv.disconnect_volume(self.slot_mgr)
+        mock_names.assert_called_once()
 
     @mock.patch('nova_powervm.virt.powervm.volume.npiv.NPIVVolumeAdapter.'
                 '_remove_maps_for_fabric')
     def test_disconnect_volume_no_op(self, mock_remove_maps):
         """Tests that when the task state is not set, connections are left."""
         # Invoke
-        self.vol_drv.disconnect_volume()
+        self.vol_drv.instance.task_state = None
+        self.vol_drv.instance.host = None
+
+        self.vol_drv.disconnect_volume(self.slot_mgr)
 
         # Verify
         self.assertEqual(0, mock_remove_maps.call_count)
@@ -200,9 +269,10 @@ class TestNPIVAdapter(test_vol.TestVolumeAdapter):
     def test_disconnect_volume_no_op_other_state(self):
         """Tests that the deletion doesn't go through on certain states."""
         self.vol_drv.instance.task_state = task_states.RESUMING
+        self.vol_drv.instance.host = CONF.host
 
         # Invoke
-        self.vol_drv.disconnect_volume()
+        self.vol_drv.disconnect_volume(self.slot_mgr)
         self.assertEqual(0, self.adpt.read.call_count)
 
     def test_connect_volume_no_map(self):
@@ -220,13 +290,13 @@ class TestNPIVAdapter(test_vol.TestVolumeAdapter):
         mock_vios.vfc_mappings = [mock_mapping]
 
         # Invoke
-        self.vol_drv.connect_volume()
+        self.vol_drv.connect_volume(self.slot_mgr)
 
     def test_min_xags(self):
         xags = self.vol_drv.min_xags()
         self.assertEqual(2, len(xags))
-        self.assertIn(pvm_vios.VIOS.xags.STORAGE, xags)
-        self.assertIn(pvm_vios.VIOS.xags.FC_MAPPING, xags)
+        self.assertIn(pvm_const.XAG.VIO_STOR, xags)
+        self.assertIn(pvm_const.XAG.VIO_FMAP, xags)
 
     @mock.patch('nova_powervm.virt.powervm.volume.npiv.NPIVVolumeAdapter.'
                 '_get_fabric_meta')
@@ -448,48 +518,138 @@ class TestNPIVAdapter(test_vol.TestVolumeAdapter):
             [('11', 'AA BB'), ('22', 'CC DD')],
             [('33', 'EE FF')]]
 
+        vios_wraps = pvm_vios.VIOS.wrap(tju.load_file(VIOS_FEED))
+        vios1_w = vios_wraps[0]
+
         def mock_client_adpt(slot):
             return mock.Mock(client_adapter=mock.Mock(lpar_slot_num=slot))
 
         mock_find_vios_for_vfc_wwpns.side_effect = [
-            (None, mock_client_adpt(1)), (None, mock_client_adpt(2)),
-            (None, mock_client_adpt(3))]
+            (vios1_w, mock_client_adpt(1)), (vios1_w, mock_client_adpt(2)),
+            (vios1_w, mock_client_adpt(3))]
 
         # Execute the test
         mig_data = {}
         self.vol_drv.pre_live_migration_on_source(mig_data)
 
-        self.assertEqual([1, 2], mig_data.get('npiv_fabric_slots_A'))
-        self.assertEqual([3], mig_data.get('npiv_fabric_slots_B'))
+        self.assertEqual('[1, 2]', mig_data.get('src_npiv_fabric_slots_A'))
+        self.assertEqual('[3]', mig_data.get('src_npiv_fabric_slots_B'))
+        vios_peer_slots = jsonutils.loads(mig_data.get('src_vios_peer_slots'))
+        self.assertItemsEqual([[1, 2, 3]], vios_peer_slots)
+        # Ensure only string data is placed in the dict.
+        for key in mig_data:
+            self.assertEqual(str, type(mig_data[key]))
+
+    @mock.patch('pypowervm.tasks.vfc_mapper.find_vios_for_vfc_wwpns')
+    @mock.patch('nova_powervm.virt.powervm.volume.npiv.NPIVVolumeAdapter.'
+                '_get_fabric_meta')
+    @mock.patch('nova_powervm.virt.powervm.volume.npiv.NPIVVolumeAdapter.'
+                '_fabric_names')
+    def test_pre_live_migration_on_source_dual_vios(
+            self, mock_fabric_names, mock_get_fabric_meta,
+            mock_find_vios_for_vfc_wwpns):
+        mock_fabric_names.return_value = ['A', 'B']
+        mock_get_fabric_meta.side_effect = [
+            [('11', 'AA BB'), ('22', 'CC DD')],
+            [('33', 'EE FF'), ('44', 'GG HH')]]
+
+        vios_wraps = pvm_vios.VIOS.wrap(tju.load_file(VIOS_FEED_2))
+        vios1_w = vios_wraps[0]
+        vios2_w = vios_wraps[1]
+
+        def mock_client_adpt(slot):
+            return mock.Mock(client_adapter=mock.Mock(lpar_slot_num=slot))
+
+        mock_find_vios_for_vfc_wwpns.side_effect = [
+            (vios1_w, mock_client_adpt(1)), (vios1_w, mock_client_adpt(2)),
+            (vios2_w, mock_client_adpt(3)), (vios2_w, mock_client_adpt(4))]
+
+        # Execute the test
+        mig_data = {}
+        self.vol_drv.pre_live_migration_on_source(mig_data)
+
+        self.assertEqual('[1, 2]', mig_data.get('src_npiv_fabric_slots_A'))
+        self.assertEqual('[3, 4]', mig_data.get('src_npiv_fabric_slots_B'))
+        vios_peer_slots = jsonutils.loads(mig_data.get('src_vios_peer_slots'))
+        self.assertItemsEqual([[1, 2], [3, 4]], vios_peer_slots)
+        # Ensure only string data is placed in the dict.
+        for key in mig_data:
+            self.assertEqual(str, type(mig_data[key]))
+
+        # test unequal mapping across 2 VIOS
+        mock_get_fabric_meta.side_effect = [
+            [('11', 'AA BB'), ('22', 'CC DD')],
+            [('33', 'EE FF')]]
+
+        mock_find_vios_for_vfc_wwpns.side_effect = [
+            (vios1_w, mock_client_adpt(1)), (vios2_w, mock_client_adpt(2)),
+            (vios1_w, mock_client_adpt(3))]
+
+        # Execute the test
+        mig_data = {}
+        self.vol_drv.pre_live_migration_on_source(mig_data)
+
+        self.assertEqual('[1, 2]', mig_data.get('src_npiv_fabric_slots_A'))
+        self.assertEqual('[3]', mig_data.get('src_npiv_fabric_slots_B'))
+        vios_peer_slots = jsonutils.loads(mig_data.get('src_vios_peer_slots'))
+        self.assertItemsEqual([[1, 3], [2]], vios_peer_slots)
+        # Ensure only string data is placed in the dict.
+        for key in mig_data:
+            self.assertEqual(str, type(mig_data[key]))
 
     @mock.patch('pypowervm.tasks.vfc_mapper.'
                 'build_migration_mappings_for_fabric')
     @mock.patch('nova_powervm.virt.powervm.volume.npiv.NPIVVolumeAdapter.'
                 '_fabric_names')
+    def test_pre_live_migration_on_destination_legacy(
+            self, mock_fabric_names, mock_build_mig_map):
+        mock_fabric_names.return_value = ['A', 'B']
+
+        mig_data = {'src_npiv_fabric_slots_A': jsonutils.dumps([1, 2]),
+                    'src_npiv_fabric_slots_B': jsonutils.dumps([3])}
+
+        mock_build_mig_map.side_effect = [['a'], ['b']]
+        self.vol_drv.stg_ftsk = mock.MagicMock()
+
+        # Execute the test
+        self.vol_drv.pre_live_migration_on_destination(mig_data)
+
+        self.assertEqual('["a"]', mig_data.get('dest_npiv_fabric_mapping_A'))
+        self.assertEqual('["b"]', mig_data.get('dest_npiv_fabric_mapping_B'))
+        # Ensure only string data is placed in the dict.
+        for key in mig_data:
+            self.assertEqual(str, type(mig_data[key]))
+
+        # Order of the mappings is not important.
+        self.assertEqual(
+            {'b', 'a'},
+            set(jsonutils.loads(mig_data.get('vfc_lpm_mappings'))))
+
+        # Verify that on migration, the WWPNs are reversed.
+        self.assertEqual(2, self.vol_drv.stg_ftsk.feed.reverse.call_count)
+
+    @mock.patch('pypowervm.tasks.vfc_mapper.'
+                'build_migration_mappings')
+    @mock.patch('nova_powervm.virt.powervm.volume.npiv.NPIVVolumeAdapter.'
+                '_fabric_names')
     def test_pre_live_migration_on_destination(
             self, mock_fabric_names, mock_build_mig_map):
         mock_fabric_names.return_value = ['A', 'B']
 
-        src_mig_data = {'npiv_fabric_slots_A': [1, 2],
-                        'npiv_fabric_slots_B': [3]}
-        dest_mig_data = {}
+        mig_data = {'src_npiv_fabric_slots_A': jsonutils.dumps([1, 2]),
+                    'src_npiv_fabric_slots_B': jsonutils.dumps([3]),
+                    'src_vios_peer_slots': jsonutils.dumps([[1, 2, 3]])}
 
-        mock_build_mig_map.side_effect = [['a'], ['b']]
+        mock_build_mig_map.return_value = {'a', 'b'}
         self.vol_drv.stg_ftsk = mock.MagicMock()
 
         # Execute the test
-        self.vol_drv.pre_live_migration_on_destination(
-            src_mig_data, dest_mig_data)
-
-        self.assertEqual(['a'], dest_mig_data.get('npiv_fabric_mapping_A'))
-        self.assertEqual(['b'], dest_mig_data.get('npiv_fabric_mapping_B'))
+        self.vol_drv.pre_live_migration_on_destination(mig_data)
 
         # Order of the mappings is not important.
-        self.assertEqual({'b', 'a'},
-                         set(dest_mig_data.get('vfc_lpm_mappings')))
-
-        # Verify that on migration, the WWPNs are reversed.
-        self.assertEqual(2, self.vol_drv.stg_ftsk.feed.reverse.call_count)
+        self.assertEqual(
+            set({'b', 'a'}),
+            set(jsonutils.loads(mig_data.get('vfc_lpm_mappings'))))
 
     def test_set_fabric_meta(self):
         port_map = [('1', 'aa AA'), ('2', 'bb BB'),
@@ -526,3 +686,6 @@ class TestNPIVAdapter(test_vol.TestVolumeAdapter):
         self.vol_drv.instance.system_metadata = system_meta
         fabric_meta = self.vol_drv._get_fabric_meta('A')
         self.assertEqual(fabric_meta, expected)
+
+    def test_vol_type(self):
+        self.assertEqual('npiv', self.vol_drv.vol_type())
