@@ -19,6 +19,7 @@ import mock
 from nova_powervm import conf as cfg
 from nova_powervm.tests.virt.powervm.volume import test_driver as test_vol
 from nova_powervm.virt.powervm import exception as p_exc
+from nova_powervm.virt.powervm.volume import volume
 from nova_powervm.virt.powervm.volume import vscsi
 
 from pypowervm import const as pvm_const
@@ -78,8 +79,8 @@ class BaseVSCSITest(test_vol.TestVolumeAdapter):
             # that would otherwise need to be mocked).
             mock_getter.return_value = self.feed
 
-            return vscsi.VscsiVolumeAdapter(self.adpt, 'host_uuid', mock_inst,
-                                            con_info)
+            return vscsi.PVVscsiFCVolumeAdapter(self.adpt, 'host_uuid',
+                                                mock_inst, con_info)
         self.vol_drv = init_vol_adpt()
 
 
@@ -98,7 +99,7 @@ class TestVSCSIAdapter(BaseVSCSITest):
         self.slot_mgr = mock.Mock()
         self.slot_mgr.build_map.get_vscsi_slot.return_value = 62, 'the_lua'
 
-    @mock.patch('pypowervm.tasks.hdisk.lua_recovery')
+    @mock.patch('pypowervm.tasks.hdisk.discover_hdisk')
     @mock.patch('pypowervm.tasks.storage.find_stale_lpars')
     def test_pre_live_migration(self, mock_fsl, mock_discover):
         # The mock return values
@@ -124,7 +125,7 @@ class TestVSCSIAdapter(BaseVSCSITest):
 
         # Bad path.  udid not found
         # Run the method - this should produce a warning
-        with self.assertLogs(vscsi.__name__, 'WARNING'):
+        with self.assertLogs(volume.__name__, 'WARNING'):
             self.vol_drv._cleanup_volume(None)
 
         # Good path
@@ -166,9 +167,9 @@ class TestVSCSIAdapter(BaseVSCSITest):
 
     @mock.patch('pypowervm.tasks.scsi_mapper.add_map')
     @mock.patch('pypowervm.tasks.scsi_mapper.build_vscsi_mapping')
-    @mock.patch('pypowervm.tasks.hdisk.lua_recovery')
+    @mock.patch('pypowervm.tasks.hdisk.discover_hdisk')
     @mock.patch('nova_powervm.virt.powervm.vm.get_vm_id')
-    def test_connect_volume_rebuild_new_vio(self, mock_get_vm_id, mock_lua,
+    def test_connect_volume_rebuild_new_vio(self, mock_get_vm_id, mock_disc,
                                             mock_build_map, mock_add_map):
         """Check if bad slot map.
 
@@ -181,7 +182,7 @@ class TestVSCSIAdapter(BaseVSCSITest):
         # Set up that the device is on the VIOS
         mock_get_vm_id.return_value = 'partition_id'
         mock_build_map.side_effect = Exception
-        mock_lua.return_value = (
+        mock_disc.return_value = (
             hdisk.LUAStatus.DEVICE_AVAILABLE, 'devname', 'udid')
 
         # Run the method.  It will fail only because there isn't a second
@@ -194,17 +195,17 @@ class TestVSCSIAdapter(BaseVSCSITest):
 
     @mock.patch('pypowervm.tasks.scsi_mapper.add_map')
     @mock.patch('pypowervm.tasks.scsi_mapper.build_vscsi_mapping')
-    @mock.patch('pypowervm.tasks.hdisk.lua_recovery')
+    @mock.patch('pypowervm.tasks.hdisk.discover_hdisk')
     @mock.patch('nova_powervm.virt.powervm.vm.get_vm_id')
-    def test_connect_volume(self, mock_get_vm_id, mock_lua_recovery,
+    def test_connect_volume(self, mock_get_vm_id, mock_disc_hdisk,
                             mock_build_map, mock_add_map):
         # The mock return values
-        mock_lua_recovery.return_value = (
+        mock_disc_hdisk.return_value = (
             hdisk.LUAStatus.DEVICE_AVAILABLE, 'devname', 'udid')
         mock_get_vm_id.return_value = 'partition_id'
 
         def build_map_func(host_uuid, vios_w, lpar_uuid, pv,
-                           lpar_slot_num=None, lua=None):
+                           lpar_slot_num=None, lua=None, target_name=None):
             self.assertEqual('host_uuid', host_uuid)
             self.assertIsInstance(vios_w, pvm_vios.VIOS)
             self.assertEqual('1234', lpar_uuid)
@@ -226,8 +227,8 @@ class TestVSCSIAdapter(BaseVSCSITest):
     @mock.patch('pypowervm.tasks.scsi_mapper.add_map')
     @mock.patch('pypowervm.tasks.scsi_mapper.build_vscsi_mapping')
     @mock.patch('pypowervm.tasks.hdisk.discover_hdisk')
-    @mock.patch('nova_powervm.virt.powervm.volume.vscsi.VscsiVolumeAdapter.'
-                '_validate_vios_on_connection')
+    @mock.patch('nova_powervm.virt.powervm.volume.vscsi.PVVscsiFCVolumeAdapter'
+                '._validate_vios_on_connection')
     @mock.patch('nova_powervm.virt.powervm.vm.get_vm_id')
     def test_connect_volume_no_update(
         self, mock_get_vm_id, mock_validate_vioses, mock_disc_hdisk,
@@ -250,17 +251,14 @@ class TestVSCSIAdapter(BaseVSCSITest):
         self.assertEqual(1, mock_disc_hdisk.call_count)
 
     @mock.patch('pypowervm.tasks.hdisk.build_itls')
-    @mock.patch('pypowervm.tasks.hdisk.lua_recovery')
     @mock.patch('pypowervm.tasks.scsi_mapper.add_vscsi_mapping')
-    @mock.patch('nova_powervm.virt.powervm.volume.vscsi.VscsiVolumeAdapter.'
-                '_validate_vios_on_connection')
+    @mock.patch('nova_powervm.virt.powervm.volume.vscsi.PVVscsiFCVolumeAdapter'
+                '._validate_vios_on_connection')
     @mock.patch('nova_powervm.virt.powervm.vm.get_vm_id')
     def test_connect_volume_to_initiators(
         self, mock_get_vm_id, mock_validate_vioses, mock_add_vscsi_mapping,
-        mock_lua_recovery, mock_build_itls):
+        mock_build_itls):
         """Tests that the connect w/out initiators throws errors."""
-        mock_lua_recovery.return_value = (
-            hdisk.LUAStatus.DEVICE_AVAILABLE, 'devname', 'udid')
         mock_get_vm_id.return_value = 'partition_id'
 
         mock_instance = mock.Mock()
@@ -449,7 +447,7 @@ class TestVSCSIAdapter(BaseVSCSITest):
         self.assertEqual(0, mock_remove_maps.call_count)
         self.assertEqual(0, self.ft_fx.patchers['update'].mock.call_count)
 
-    @mock.patch('nova_powervm.virt.powervm.vios.get_physical_wwpns')
+    @mock.patch('pypowervm.tasks.partition.get_physical_wwpns')
     def test_wwpns(self, mock_vio_wwpns):
         mock_vio_wwpns.return_value = ['aa', 'bb']
 
@@ -550,7 +548,7 @@ class TestVSCSIAdapterMultiVIOS(BaseVSCSITest):
         mock_vm_id.return_value = 'partition_id'
 
         def build_map_func(host_uuid, vios_w, lpar_uuid, pv,
-                           lpar_slot_num=None, lua=None):
+                           lpar_slot_num=None, lua=None, target_name=None):
             self.assertEqual('host_uuid', host_uuid)
             self.assertIsInstance(vios_w, pvm_vios.VIOS)
             self.assertEqual('1234', lpar_uuid)
@@ -571,7 +569,7 @@ class TestVSCSIAdapterMultiVIOS(BaseVSCSITest):
         # Two of the calls are for the slots, two are for the add mappings
         self.assertEqual(2, self.ft_fx.patchers['update'].mock.call_count)
 
-    @mock.patch('pypowervm.tasks.hdisk.lua_recovery')
+    @mock.patch('pypowervm.tasks.hdisk.discover_hdisk')
     @mock.patch('pypowervm.tasks.storage.find_stale_lpars')
     def test_pre_live_migration_multi_vios(self, mock_fsl, mock_discover):
         # The mock return values
