@@ -20,6 +20,7 @@ from oslo_log import log as logging
 from nova import exception as nova_exc
 from pypowervm import const as pvm_const
 from pypowervm import exceptions as pvm_exc
+from pypowervm.tasks import partition as pvm_tpar
 from pypowervm.tasks import scsi_mapper as tsk_map
 from pypowervm.tasks import storage as tsk_stg
 from pypowervm.wrappers import managed_system as pvm_ms
@@ -31,7 +32,6 @@ from nova_powervm.virt.powervm.disk import driver as disk_dvr
 from nova_powervm.virt.powervm import exception as npvmex
 from nova_powervm.virt.powervm.i18n import _LE
 from nova_powervm.virt.powervm.i18n import _LI
-from nova_powervm.virt.powervm import vios
 from nova_powervm.virt.powervm import vm
 
 
@@ -40,8 +40,8 @@ CONF = cfg.CONF
 
 
 class LocalStorage(disk_dvr.DiskAdapter):
-    def __init__(self, connection):
-        super(LocalStorage, self).__init__(connection)
+    def __init__(self, adapter, host_uuid):
+        super(LocalStorage, self).__init__(adapter, host_uuid)
 
         # Query to get the Volume Group UUID
         self.vg_name = CONF.powervm.volume_group_name
@@ -123,9 +123,8 @@ class LocalStorage(disk_dvr.DiskAdapter):
 
         # Ensure we have a transaction manager.
         if stg_ftsk is None:
-            stg_ftsk = vios.build_tx_feed_task(
-                self.adapter, self.host_uuid, name='localdisk',
-                xag=[pvm_const.XAG.VIO_SMAP])
+            stg_ftsk = pvm_tpar.build_active_vio_feed_task(
+                self.adapter, name='localdisk', xag=[pvm_const.XAG.VIO_SMAP])
 
         # Build the match function
         match_func = tsk_map.gen_match_func(pvm_stg.VDisk, prefixes=disk_type)
@@ -196,11 +195,9 @@ class LocalStorage(disk_dvr.DiskAdapter):
         # resize the disk, create a new partition, etc...
         # If the image is bigger than disk, API should make the disk big
         # enough to support the image (up to 1 Gb boundary).
-        vdisk, f_wrap = tsk_stg.upload_new_vdisk(
+        return tsk_stg.upload_new_vdisk(
             self.adapter, self._vios_uuid, self.vg_uuid, stream, vol_name,
-            image_meta.size, d_size=disk_bytes)
-
-        return vdisk
+            image_meta.size, d_size=disk_bytes)[0]
 
     def connect_disk(self, context, instance, disk_info, stg_ftsk=None):
         """Connects the disk image to the Virtual Machine.
@@ -221,9 +218,8 @@ class LocalStorage(disk_dvr.DiskAdapter):
 
         # Ensure we have a transaction manager.
         if stg_ftsk is None:
-            stg_ftsk = vios.build_tx_feed_task(
-                self.adapter, self.host_uuid, name='localdisk',
-                xag=[pvm_const.XAG.VIO_SMAP])
+            stg_ftsk = pvm_tpar.build_active_vio_feed_task(
+                self.adapter, name='localdisk', xag=[pvm_const.XAG.VIO_SMAP])
 
         def add_func(vios_w):
             LOG.info(_LI("Adding logical volume disk connection between VM "
