@@ -44,6 +44,8 @@ from pypowervm.helpers import vios_busy as vio_hlp
 from pypowervm.tasks import memory as pvm_mem
 from pypowervm.tasks import partition as pvm_par
 from pypowervm.tasks import power as pvm_pwr
+from pypowervm.tasks import scsi_mapper as pvm_smap
+from pypowervm.tasks import storage as pvm_stor
 from pypowervm.tasks import vterm as pvm_vterm
 from pypowervm import util as pvm_util
 from pypowervm.wrappers import base_partition as pvm_bp
@@ -65,7 +67,6 @@ from nova_powervm.virt.powervm.tasks import network as tf_net
 from nova_powervm.virt.powervm.tasks import slot as tf_slot
 from nova_powervm.virt.powervm.tasks import storage as tf_stg
 from nova_powervm.virt.powervm.tasks import vm as tf_vm
-from nova_powervm.virt.powervm import vios
 from nova_powervm.virt.powervm import vm
 from nova_powervm.virt.powervm import volume as vol_attach
 
@@ -73,11 +74,13 @@ LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
 
 # Defines, for all cinder volume types, which volume driver to use.  Currently
-# only supports Fibre Channel, which has multiple options for connections.
-# The connection strategy is defined above.
+# only supports Fibre Channel, which has multiple options for connections, and
+# iSCSI.
 VOLUME_DRIVER_MAPPINGS = {
     'fibre_channel': vol_attach.FC_STRATEGY_MAPPING[
-        CONF.powervm.fc_attach_strategy]
+        CONF.powervm.fc_attach_strategy.lower()],
+    'iscsi': vol_attach.NETWORK_STRATEGY_MAPPING[
+        CONF.powervm.network_attach_strategy.lower()],
 }
 
 DISK_ADPT_NS = 'nova_powervm.virt.powervm.disk'
@@ -102,7 +105,9 @@ class PowerVMDriver(driver.ComputeDriver):
     capabilities = {
         "has_imagecache": False,
         "supports_recreate": True,
-        "supports_migrate_to_same_host": False
+        "supports_migrate_to_same_host": False,
+        "supports_attach_interface": True,
+        "supports_device_tagging": False
     }
 
     def __init__(self, virtapi):
@@ -123,15 +128,16 @@ class PowerVMDriver(driver.ComputeDriver):
         self._get_adapter()
         # First need to resolve the managed host UUID
         self._get_host_uuid()
-        # Get the management partition's UUID
-        self.mp_uuid = pvm_par.get_this_partition(self.adapter).uuid
-        LOG.debug("Driver found compute partition UUID of: %s" % self.mp_uuid)
 
         # Make sure the Virtual I/O Server(s) are available.
-        vios.validate_vios_ready(self.adapter, self.host_uuid)
+        pvm_par.validate_vios_ready(self.adapter)
+
+        # Do a scrub of the I/O plane to make sure the system is in good shape
+        LOG.info(_LI("Clearing stale I/O connections on driver init."))
+        pvm_stor.ComprehensiveScrub(self.adapter).execute()
 
         # Initialize the disk adapter.  Sets self.disk_dvr
-        self._get_disk_adapter()
+        self._setup_disk_adapter()
         self.image_api = image.API()
 
         self._setup_rebuild_store()
@@ -171,16 +177,14 @@ class PowerVMDriver(driver.ComputeDriver):
         eh = NovaEventHandler(self)
         self.session.get_event_listener().subscribe(eh)
 
-    def _get_disk_adapter(self):
-        conn_info = {'adapter': self.adapter, 'host_uuid': self.host_uuid,
-                     'mp_uuid': self.mp_uuid}
-
+    def _setup_disk_adapter(self):
+        """Set up the nova ephemeral disk adapter."""
         self.disk_dvr = importutils.import_object_ns(
             DISK_ADPT_NS, DISK_ADPT_MAPPINGS[CONF.powervm.disk_driver.lower()],
-            conn_info)
+            self.adapter, self.host_uuid)
 
     def _setup_rebuild_store(self):
-        """Setup the store for remote restart objects."""
+        """Set up the store for remote restart objects."""
         store = CONF.powervm.nvram_store.lower()
         if store != 'none':
             self.store_api = importutils.import_object(
@@ -247,7 +251,7 @@ class PowerVMDriver(driver.ComputeDriver):
         Returns True if an instance with the supplied ID exists on
         the host, False otherwise.
         """
-        return vm.instance_exists(self.adapter, instance, self.host_uuid)
+        return vm.instance_exists(self.adapter, instance)
 
     def estimate_instance_overhead(self, instance_info):
         """Estimate the virtualization overhead required to build an instance.
@@ -391,8 +395,7 @@ class PowerVMDriver(driver.ComputeDriver):
 
         # Create the transaction manager (FeedTask) for Storage I/O.
         xag = self._get_inst_xag(instance, bdms, recreate=recreate)
-        stg_ftsk = vios.build_tx_feed_task(self.adapter, self.host_uuid,
-                                           xag=xag)
+        stg_ftsk = pvm_par.build_active_vio_feed_task(self.adapter, xag=xag)
 
         # Build the PowerVM Slot lookup map.  Only the recreate action needs
         # the volume driver iterator (to look up volumes and their client
@@ -408,8 +411,14 @@ class PowerVMDriver(driver.ComputeDriver):
         nvram_mgr = (self.nvram_mgr if self.nvram_mgr and
                      (recreate or instance.vm_state in FETCH_NVRAM_STATES)
                      else None)
-        flow_spawn.add(tf_vm.Create(self.adapter, self.host_wrapper, instance,
-                                    flavor, stg_ftsk, nvram_mgr=nvram_mgr))
+
+        # If we're recreating pass None in for the FeedTask. This will make the
+        # Create task produce a FeedTask that will be used to scrub stale
+        # adapters immediately after the LPAR is created.
+        flow_spawn.add(tf_vm.Create(
+            self.adapter, self.host_wrapper, instance, flavor,
+            stg_ftsk=(None if recreate else stg_ftsk), nvram_mgr=nvram_mgr,
+            slot_mgr=slot_mgr))
 
         # Create a flow for the IO
         flow_spawn.add(tf_net.PlugVifs(
@@ -456,7 +465,7 @@ class PowerVMDriver(driver.ComputeDriver):
             boot_type = self._get_boot_connectivity_type(
                 context, bdms, block_device_info)
             flow_spawn.add(tf_vm.UpdateIBMiSettings(
-                self.adapter, instance, self.host_uuid, boot_type))
+                self.adapter, instance, boot_type))
 
         # Save the slot map information
         flow_spawn.add(tf_slot.SaveSlotStore(instance, slot_mgr))
@@ -576,8 +585,8 @@ class PowerVMDriver(driver.ComputeDriver):
 
             # Create the transaction manager (FeedTask) for Storage I/O.
             xag = self._get_inst_xag(instance, bdms)
-            stg_ftsk = vios.build_tx_feed_task(self.adapter, self.host_uuid,
-                                               xag=xag)
+            stg_ftsk = pvm_par.build_active_vio_feed_task(self.adapter,
+                                                          xag=xag)
 
             # Build the PowerVM Slot lookup map.
             slot_mgr = slot.build_slot_mgr(instance, self.store_api)
@@ -590,8 +599,10 @@ class PowerVMDriver(driver.ComputeDriver):
 
             # Add the disconnect/deletion of the vOpt to the transaction
             # manager.
-            flow.add(tf_stg.DeleteVOpt(self.adapter, self.host_uuid, instance,
-                                       pvm_inst_uuid, stg_ftsk=stg_ftsk))
+            if configdrive.required_by(instance):
+                flow.add(tf_stg.DeleteVOpt(
+                    self.adapter, self.host_uuid, instance, pvm_inst_uuid,
+                    stg_ftsk=stg_ftsk))
 
             # Determine if there are volumes to disconnect.  If so, remove each
             # volume (within the transaction manager)
@@ -611,6 +622,24 @@ class PowerVMDriver(driver.ComputeDriver):
                     destroy_disk_task = tf_stg.DeleteDisk(
                         self.disk_dvr, context, instance)
 
+            # It's possible that volume disconnection may have failed for disks
+            # which had been removed from the VIOS by the storage back end
+            # (e.g. if we're destroying an evacuated instance).  In that case,
+            # the storage is already gone, but we had no way to identify its
+            # mappings because no disk name/UDID.  We now remove such mappings
+            # based on their association with the LPAR ID.
+            def _rm_vscsi_maps(vwrap):
+                removals = pvm_smap.remove_maps(vwrap, pvm_inst_uuid)
+                if removals:
+                    LOG.warning(_LW("Removing %(num_maps)d storage-less VSCSI "
+                                    "mappings associated with LPAR ID "
+                                    "%(lpar_uuid)s from VIOS %(vios_name)s."),
+                                {'num_maps': len(removals),
+                                 'lpar_uuid': pvm_inst_uuid,
+                                 'vios_name': vwrap.name})
+                return removals
+            stg_ftsk.add_functor_subtask(_rm_vscsi_maps)
+
             # Add the transaction manager flow to the end of the 'storage
             # connection' tasks.  This will run all the disconnection ops
             # in parallel
@@ -656,6 +685,7 @@ class PowerVMDriver(driver.ComputeDriver):
                             instance=instance)
                 return
             else:
+                LOG.exception(e)
                 # Convert to a Nova exception
                 raise exception.InstanceTerminationFailure(
                     reason=six.text_type(e))
@@ -753,7 +783,7 @@ class PowerVMDriver(driver.ComputeDriver):
         # for each volume attached to the instance, against the destination
         # host.  If the migration failed, then the VM is probably not on
         # the destination host.
-        if not vm.instance_exists(self.adapter, instance, self.host_uuid):
+        if not vm.instance_exists(self.adapter, instance):
             LOG.info(_LI('During volume detach, the instance was not found'
                          ' on this host.'), instance=instance)
 
@@ -943,7 +973,7 @@ class PowerVMDriver(driver.ComputeDriver):
         """
         self._log_operation(reboot_type + ' reboot', instance)
         force_immediate = reboot_type == 'HARD'
-        entry = vm.get_instance_wrapper(self.adapter, instance, self.host_uuid)
+        entry = vm.get_instance_wrapper(self.adapter, instance)
         if entry.state != pvm_bp.LPARState.NOT_ACTIVATED:
             pvm_pwr.power_off(entry, self.host_uuid, restart=True,
                               force_immediate=force_immediate)
@@ -969,11 +999,8 @@ class PowerVMDriver(driver.ComputeDriver):
             a driver that manages only one node can safely ignore this
         :return: Dictionary describing resources
         """
-
-        resp = self.adapter.read(pvm_ms.System.schema_type,
-                                 root_id=self.host_uuid)
-        if resp:
-            self.host_wrapper = pvm_ms.System.wrap(resp.entry)
+        # Do this here so it refreshes each time this method is called.
+        self.host_wrapper = pvm_ms.System.get(self.adapter)[0]
         # Get host information
         data = pvm_host.build_host_resource_from_ms(self.host_wrapper)
 
@@ -1107,15 +1134,14 @@ class PowerVMDriver(driver.ComputeDriver):
 
         # The WWPNs in case of FC connection.
         if vol_drv is not None:
-            # Override the host name.
-            # TODO(IBM) See if there is a way to support a FC host name that
-            # is independent of overall host name.
-            connector['host'] = vol_drv.host_name()
 
-            # Set the WWPNs
-            wwpn_list = vol_drv.wwpns()
-            if wwpn_list is not None:
-                connector["wwpns"] = wwpn_list
+            if CONF.powervm.volume_adapter.lower() == "fibre_channel":
+                # Set the WWPNs
+                wwpn_list = vol_drv.wwpns()
+                if wwpn_list is not None:
+                    connector["wwpns"] = wwpn_list
+            connector['host'] = vol_drv.host_name()
+            connector['initiator'] = vol_drv.host_name()
         return connector
 
     def migrate_disk_and_power_off(self, context, instance, dest,
@@ -1172,8 +1198,8 @@ class PowerVMDriver(driver.ComputeDriver):
         if bdms:
             # Create the transaction manager (FeedTask) for Storage I/O.
             xag = self._get_inst_xag(instance, bdms)
-            stg_ftsk = vios.build_tx_feed_task(self.adapter, self.host_uuid,
-                                               xag=xag)
+            stg_ftsk = pvm_par.build_active_vio_feed_task(self.adapter,
+                                                          xag=xag)
 
             # Get the slot map.  This is so we build the client
             # adapters in the same slots.
@@ -1196,8 +1222,7 @@ class PowerVMDriver(driver.ComputeDriver):
         # easy to see the VM is being migrated from pvmctl.  We use the resize
         # name so we don't destroy it on a revert when it's on the same host.
         new_name = self._gen_resize_name(instance, same_host=same_host)
-        flow.add(tf_vm.Rename(self.adapter, self.host_uuid, instance,
-                              new_name))
+        flow.add(tf_vm.Rename(self.adapter, instance, new_name))
         try:
             tf_eng.run(flow)
         except Exception as e:
@@ -1270,8 +1295,8 @@ class PowerVMDriver(driver.ComputeDriver):
         if bdms or not same_host:
             # Create the transaction manager (FeedTask) for Storage I/O.
             xag = self._get_inst_xag(instance, bdms)
-            stg_ftsk = vios.build_tx_feed_task(self.adapter, self.host_uuid,
-                                               xag=xag)
+            stg_ftsk = pvm_par.build_active_vio_feed_task(self.adapter,
+                                                          xag=xag)
             # We need the slot manager
             # a) If migrating to a different host: to restore the proper slots;
             # b) If adding/removing block devices, to register the slots.
@@ -1292,7 +1317,7 @@ class PowerVMDriver(driver.ComputeDriver):
             # Create the LPAR
             flow.add(tf_vm.Create(self.adapter, self.host_wrapper, instance,
                                   instance.flavor, stg_ftsk,
-                                  nvram_mgr=self.nvram_mgr))
+                                  nvram_mgr=self.nvram_mgr, slot_mgr=slot_mgr))
 
             # Create a flow for the network IO
             flow.add(tf_net.PlugVifs(self.virtapi, self.adapter, instance,
@@ -1344,7 +1369,7 @@ class PowerVMDriver(driver.ComputeDriver):
         if same_host:
             # This was a local resize, don't delete our only VM!
             self._log_operation('confirm resize', instance)
-            vm.rename(self.adapter, self.host_uuid, instance, instance.name)
+            vm.rename(self.adapter, instance, instance.name)
             return
 
         # Confirming the migrate means we need to delete source VM.
@@ -1436,8 +1461,8 @@ class PowerVMDriver(driver.ComputeDriver):
         return mig.check_destination(context, src_compute_info,
                                      dst_compute_info)
 
-    def check_can_live_migrate_destination_cleanup(self, context,
-                                                   dest_check_data):
+    def cleanup_live_migration_destination_check(self, context,
+                                                 dest_check_data):
         """Do required cleanup on dest host after check_can_live_migrate calls
 
         :param context: security context
@@ -1756,8 +1781,27 @@ class PowerVMDriver(driver.ComputeDriver):
 
         # Build the connection to the VNC.
         host = CONF.vnc.vncserver_proxyclient_address
-        port = pvm_vterm.open_remotable_vnc_vterm(
-            self.adapter, lpar_uuid, host, vnc_path=lpar_uuid)
+        use_x509_auth = CONF.powervm.vnc_use_x509_auth
+        ca_certs = CONF.powervm.vnc_ca_certs
+        server_key = CONF.powervm.vnc_server_key
+        server_cert = CONF.powervm.vnc_server_cert
+        try:
+            # Open up a remote vterm with the host and certificates configured
+            # This will only use TLS if the use_x509_auth is set to True
+            port = pvm_vterm.open_remotable_vnc_vterm(
+                self.adapter, lpar_uuid, host, vnc_path=lpar_uuid,
+                use_x509_auth=use_x509_auth, ca_certs=ca_certs,
+                server_cert=server_cert, server_key=server_key)
+        except Exception as err:
+            # If the LPAR was not found, then give a more descriptive message
+            if isinstance(err, pvm_exc.HttpError):
+                if err.response.status == 404:
+                    msg = _("Unable to open console since virtual machine "
+                            "%s does not exist.") % instance['display_name']
+                    raise exception.ConsoleTypeUnavailable(message=msg)
+            # Otherwise wrapper the error in an exception that can be handled
+            raise exception.ConsoleTypeUnavailable(
+                message=_("Unable to open console.  Error is: %s") % err)
 
         # Note that the VNC viewer will wrap the internal_access_path with
         # the HTTP content.
@@ -1788,10 +1832,8 @@ class PowerVMDriver(driver.ComputeDriver):
                       {'inst': instance.name,
                        'xags': ','.join(xags)})
             return list(xags)
-
         # If we have any volumes, add the volumes required mapping XAGs.
-        adp_type = vol_attach.FC_STRATEGY_MAPPING[
-            CONF.powervm.fc_attach_strategy.lower()]
+        adp_type = VOLUME_DRIVER_MAPPINGS[CONF.powervm.volume_adapter]
         vol_cls = importutils.import_class(adp_type)
         xags.update(set(vol_cls.min_xags()))
         LOG.debug('Instance XAGs for VM %(inst)s is %(xags)s.',
@@ -1820,8 +1862,7 @@ class PowerVMDriver(driver.ComputeDriver):
                  the adapter based on the connection-type of
                  connection_info.
         """
-        adp_type = vol_attach.FC_STRATEGY_MAPPING[
-            CONF.powervm.fc_attach_strategy]
+        adp_type = VOLUME_DRIVER_MAPPINGS[CONF.powervm.volume_adapter]
         vol_cls = importutils.import_class(adp_type)
         if conn_info:
             LOG.debug('Volume Adapter returned for connection_info=%s' %
@@ -1847,14 +1888,20 @@ class PowerVMDriver(driver.ComputeDriver):
         if self._is_booted_from_volume(block_device_info) and bdms is not None:
             for bdm in bdms:
                 if bdm.get('boot_index') == 0:
-                    conn_info = bdm.get('connection_info')
-                    connectivity_type = conn_info['data']['connection-type']
-                    boot_conn_type = ('vscsi' if connectivity_type ==
-                                      'pv_vscsi' else connectivity_type)
-                    return boot_conn_type
+                    return self._get_connectivity_type(bdm)
         else:
             return boot_conn_type
 
+    def _get_connectivity_type(self, bdm):
+        conn_info = bdm.get('connection_info')
+        if 'connection-type' in conn_info['data']:
+            connectivity_type = conn_info['data']['connection-type']
+            boot_conn_type = ('vscsi' if connectivity_type == 'pv_vscsi' else
+                              connectivity_type)
+        elif 'driver_volume_type' in conn_info['data']:
+            boot_conn_type = conn_info['data']['driver_volume_type']
+        return boot_conn_type
+
 
 class NovaEventHandler(pvm_apt.RawEventHandler):
     """Used to receive and handle events from PowerVM."""
