@@ -21,22 +21,19 @@ from nova.virt import configdrive
 import os
 from taskflow import task
 
-from oslo_concurrency import lockutils
 from oslo_log import log as logging
 
 from pypowervm import const as pvm_const
 from pypowervm.tasks import scsi_mapper as tsk_map
 from pypowervm.tasks import storage as tsk_stg
+from pypowervm.tasks import vopt as tsk_vopt
 from pypowervm import util as pvm_util
 from pypowervm.utils import transaction as pvm_tx
-from pypowervm.wrappers import base_partition as pvm_bp
 from pypowervm.wrappers import storage as pvm_stg
 from pypowervm.wrappers import virtual_io_server as pvm_vios
 
 from nova_powervm import conf as cfg
-from nova_powervm.virt.powervm import exception as npvmex
 from nova_powervm.virt.powervm.i18n import _LI
-from nova_powervm.virt.powervm.i18n import _LW
 from nova_powervm.virt.powervm import vm
 
 LOG = logging.getLogger(__name__)
@@ -48,10 +45,6 @@ _LLA_SUBNET = "fe80::/64"
 
 class ConfigDrivePowerVM(object):
 
-    _cur_vios_uuid = None
-    _cur_vios_name = None
-    _cur_vg_uuid = None
-
     def __init__(self, adapter, host_uuid):
         """Creates the config drive manager for PowerVM.
 
@@ -61,13 +54,28 @@ class ConfigDrivePowerVM(object):
         self.adapter = adapter
         self.host_uuid = host_uuid
 
-        # The validate will use the cached static variables for the VIOS info.
-        # Once validate is done, set the class variables to the updated cache.
-        self._validate_vopt_vg()
+        # Validate that the virtual optical exists
+        self.vios_uuid, self.vg_uuid = tsk_vopt.validate_vopt_repo_exists(
+            self.adapter, CONF.powervm.vopt_media_volume_group,
+            CONF.powervm.vopt_media_rep_size)
 
-        self.vios_uuid = ConfigDrivePowerVM._cur_vios_uuid
-        self.vios_name = ConfigDrivePowerVM._cur_vios_name
-        self.vg_uuid = ConfigDrivePowerVM._cur_vg_uuid
+    def _sanitize_network_info(self, network_info):
+        """Will sanitize the network info for the config drive.
+
+        Newer versions of cloud-init look at the vif type information in
+        the network info and utilize it to determine what to do.  There are
+        a limited number of vif types, and it seems to be built on the idea
+        that the neutron vif type is the cloud init vif type (which is not
+        quite right).
+
+        This sanitizes the network info that gets passed into the config
+        drive to work properly with cloud-inits.
+        """
+        network_info = copy.deepcopy(network_info)
+        for vif in network_info:
+            if vif.get('type') is not 'ovs':
+                vif['type'] = 'vif'
+        return network_info
 
     def _create_cfg_dr_iso(self, instance, injected_files, network_info,
                            admin_pass=None):
@@ -88,6 +96,9 @@ class ConfigDrivePowerVM(object):
         if admin_pass is not None:
             extra_md['admin_pass'] = admin_pass
 
+        # Sanitize the vifs for the network config
+        network_info = self._sanitize_network_info(network_info)
+
         inst_md = instance_metadata.InstanceMetadata(instance,
                                                      content=injected_files,
                                                      extra_md=extra_md,
@@ -225,120 +236,6 @@ class ConfigDrivePowerVM(object):
             return tsk_stg.upload_vopt(self.adapter, self.vios_uuid, d_stream,
                                        file_name, file_size)
 
-    @lockutils.synchronized('validate_vopt')
-    def _validate_vopt_vg(self):
-        """Will ensure that the virtual optical media repository exists.
-
-        This method will connect to one of the Virtual I/O Servers on the
-        system and ensure that there is a root_vg that the optical media (which
-        is temporary) exists.
-
-        If the volume group on an I/O Server goes down (perhaps due to
-        maintenance), the system will rescan to determine if there is another
-        I/O Server that can host the request.
-
-        The very first invocation may be expensive.  It may also be expensive
-        to call if a Virtual I/O Server unexpectantly goes down.
-
-        If there are no Virtual I/O Servers that can support the media, then
-        an exception will be thrown.
-        """
-
-        # If our static variables were set, then we should validate that the
-        # repo is still running.  Otherwise, we need to reset the variables
-        # (as it could be down for maintenance).
-        if ConfigDrivePowerVM._cur_vg_uuid is not None:
-            vio_uuid = ConfigDrivePowerVM._cur_vios_uuid
-            vg_uuid = ConfigDrivePowerVM._cur_vg_uuid
-            try:
-                vg_wrap = pvm_stg.VG.get(self.adapter, uuid=vg_uuid,
-                                         parent_type=pvm_vios.VIOS,
-                                         parent_uuid=vio_uuid)
-                if vg_wrap is not None and len(vg_wrap.vmedia_repos) != 0:
-                    return
-            except Exception as exc:
-                LOG.exception(exc)
-
-            LOG.info(_LI("An error occurred querying the virtual optical "
-                         "media repository.  Attempting to re-establish "
-                         "connection with a virtual optical media repository"))
-
-        # If we're hitting this:
-        # a) It's our first time booting up;
-        # b) The previously-used Volume Group went offline (e.g. VIOS went down
-        #    for maintenance); OR
-        # c) The previously-used media repository disappeared.
-        #
-        # Since it doesn't matter which VIOS we use for the media repo, we
-        # should query all Virtual I/O Servers and see if an appropriate
-        # media repository exists.
-        vio_wraps = pvm_vios.VIOS.get(self.adapter)
-
-        # First loop through the VIOSes and their VGs to see if a media repos
-        # already exists.
-        found_vg = None
-        found_vios = None
-
-        # And in case we don't find the media repos, keep track of the VG on
-        # which we should create it.
-        conf_vg = None
-        conf_vios = None
-
-        for vio_wrap in vio_wraps:
-            # If the RMC state is not active, skip over to ensure we don't
-            # timeout
-            if vio_wrap.rmc_state != pvm_bp.RMCState.ACTIVE:
-                continue
-
-            try:
-                vg_wraps = pvm_stg.VG.get(self.adapter,
-                                          parent_type=pvm_vios.VIOS,
-                                          parent_uuid=vio_wrap.uuid)
-                for vg_wrap in vg_wraps:
-                    if len(vg_wrap.vmedia_repos) != 0:
-                        found_vg = vg_wrap
-                        found_vios = vio_wrap
-                        break
-                    # In case no media repos exists, save a pointer to the
-                    # CONFigured vopt_media_volume_group if we find it.
-                    if (conf_vg is None and vg_wrap.name ==
-                            CONF.powervm.vopt_media_volume_group):
-                        conf_vg = vg_wrap
-                        conf_vios = vio_wrap
-
-            except Exception:
-                LOG.warning(_LW('Unable to read volume groups for Virtual '
-                                'I/O Server %s'), vio_wrap.name)
-
-            # If we found it, don't keep looking
-            if found_vg:
-                break
-
-        # If we didn't find a media repos OR an appropriate volume group, raise
-        # the exception.  Since vopt_media_volume_group defaults to rootvg,
-        # which is always present, this should only happen if:
-        # a) No media repos exists on any VIOS we can see; AND
-        # b) The user specified a non-rootvg vopt_media_volume_group; AND
-        # c) The specified volume group did not exist on any VIOS.
-        if found_vg is None and conf_vg is None:
-            raise npvmex.NoMediaRepoVolumeGroupFound(
-                vol_grp=CONF.powervm.vopt_media_volume_group)
-
-        # If no media repos was found, create it.
-        if found_vg is None:
-            found_vg = conf_vg
-            found_vios = conf_vios
-            vopt_repo = pvm_stg.VMediaRepos.bld(
-                self.adapter, 'vopt', str(CONF.powervm.vopt_media_rep_size))
-            found_vg.vmedia_repos = [vopt_repo]
-            found_vg = found_vg.update()
-
-        # At this point, we know that we've successfully found or created the
-        # volume group.  Save to the static class variables.
-        ConfigDrivePowerVM._cur_vg_uuid = found_vg.uuid
-        ConfigDrivePowerVM._cur_vios_uuid = found_vios.uuid
-        ConfigDrivePowerVM._cur_vios_name = found_vios.name
-
     def dlt_vopt(self, lpar_uuid, stg_ftsk=None, remove_mappings=True):
         """Deletes the virtual optical and scsi mappings for a VM.
 
@@ -426,4 +323,6 @@ class ConfigDrivePowerVM(object):
                                        child_id=self.vg_uuid)
             tsk_stg.rm_vg_storage(pvm_stg.VG.wrap(vg_rsp), vopts=media_elems)
 
-        stg_ftsk.add_post_execute(task.FunctorTask(rm_vopt))
+        # Don't add this task if there is no media to delete (eg. config drive)
+        if media_elems:
+            stg_ftsk.add_post_execute(task.FunctorTask(rm_vopt))
