@@ -15,20 +15,25 @@
 #    under the License.
 
 from oslo_log import log as logging
+from pypowervm import const as pvm_const
+from pypowervm.tasks import partition as pvm_tpar
 from pypowervm.tasks import power
 from pypowervm.tasks import storage as pvm_stg
 import six
-from taskflow import task
 from taskflow.types import failure as task_fail
 
 from nova_powervm.virt.powervm.i18n import _LE
 from nova_powervm.virt.powervm.i18n import _LI
+from nova_powervm.virt.powervm.i18n import _LW
+from nova_powervm.virt.powervm.tasks import base as pvm_task
 from nova_powervm.virt.powervm import vm
 
+from nova.compute import task_states
+
 LOG = logging.getLogger(__name__)
 
 
-class Get(task.Task):
+class Get(pvm_task.PowerVMTask):
 
     """The task for getting a VM entry."""
 
@@ -41,29 +46,30 @@ class Get(task.Task):
         :param host_uuid: The host UUID
         :param instance: The nova instance.
         """
-        super(Get, self).__init__(name='get_lpar',
-                                  provides='lpar_wrap')
+        super(Get, self).__init__(instance, 'get_vm', provides='lpar_wrap')
         self.adapter = adapter
         self.host_uuid = host_uuid
-        self.instance = instance
 
-    def execute(self):
-        return vm.get_instance_wrapper(self.adapter, self.instance,
-                                       self.host_uuid)
+    def execute_impl(self):
+        return vm.get_instance_wrapper(self.adapter, self.instance)
 
 
-class Create(task.Task):
+class Create(pvm_task.PowerVMTask):
 
     """The task for creating a VM."""
 
-    def __init__(self, adapter, host_wrapper, instance, flavor, stg_ftsk,
-                 nvram_mgr=None):
+    def __init__(self, adapter, host_wrapper, instance, flavor, stg_ftsk=None,
+                 nvram_mgr=None, slot_mgr=None):
         """Creates the Task for creating a VM.
 
-        The revert method is not implemented because the compute manager
-        calls the driver destroy operation for spawn errors.  By not deleting
-        the lpar, it's a cleaner flow through the destroy operation and
-        accomplishes the same result.
+        The revert method only needs to do something for failed rebuilds.
+        Since the rebuild and build methods have different flows, it is
+        necessary to clean up the destination LPAR on fails during rebuild.
+
+        The revert method is not implemented for build because the compute
+        manager calls the driver destroy operation for spawn errors. By
+        not deleting the lpar, it's a cleaner flow through the destroy
+        operation and accomplishes the same result.
 
         Any stale storage associated with the new VM's (possibly recycled) ID
         will be cleaned up.  The cleanup work will be delegated to the FeedTask
@@ -75,21 +81,27 @@ class Create(task.Task):
         :param host_wrapper: The managed system wrapper
         :param instance: The nova instance.
         :param flavor: The nova flavor.
-        :param stg_ftsk: A FeedTask managing storage I/O operations.
+        :param stg_ftsk: (Optional, Default: None) A FeedTask managing storage
+                         I/O operations.  If None, one will be built locally
+                         and executed immediately. Otherwise it is the caller's
+                         responsibility to execute the FeedTask.
         :param nvram_mgr: The NVRAM manager to fetch the NVRAM from. If None,
                           the NVRAM will not be fetched.
+        :param slot_mgr: A NovaSlotManager.  Used to store/retrieve the
+                         maximum number of virtual slots for the VM.
         """
-        super(Create, self).__init__(name='crt_lpar', provides='lpar_wrap')
+        super(Create, self).__init__(
+            instance, 'crt_vm', provides='lpar_wrap')
         self.adapter = adapter
         self.host_wrapper = host_wrapper
-        self.instance = instance
         self.flavor = flavor
-        self.stg_ftsk = stg_ftsk
+        self.stg_ftsk = stg_ftsk or pvm_tpar.build_active_vio_feed_task(
+            adapter, name='create_scrubber',
+            xag={pvm_const.XAG.VIO_SMAP, pvm_const.XAG.VIO_FMAP})
         self.nvram_mgr = nvram_mgr
+        self.slot_mgr = slot_mgr
 
-    def execute(self):
-        LOG.info(_LI('Creating instance: %s'), self.instance.name)
-
+    def execute_impl(self):
         data = None
         if self.nvram_mgr is not None:
             LOG.info(_LI('Fetching NVRAM for instance %s.'),
@@ -98,13 +110,35 @@ class Create(task.Task):
             LOG.debug('NVRAM data is: %s', data, instance=self.instance)
 
         wrap = vm.crt_lpar(self.adapter, self.host_wrapper, self.instance,
-                           self.flavor, nvram=data)
+                           self.flavor, nvram=data, slot_mgr=self.slot_mgr)
         pvm_stg.add_lpar_storage_scrub_tasks([wrap.id], self.stg_ftsk,
                                              lpars_exist=True)
+        # If the stg_ftsk passed in was None and we initialized a
+        # 'create_scrubber' stg_ftsk then run it immediately. We do
+        # this because we moved the LPAR storage scrub tasks out of the
+        # build_map initialization. This was so that we could construct the
+        # build map earlier in the spawn, just before the LPAR is created.
+        # Only rebuilds should be passing in None for stg_ftsk.
+        if self.stg_ftsk.name == 'create_scrubber':
+            LOG.info(_LI('Scrubbing storage for instance %s as part of '
+                         'rebuild.'), self.instance.name,
+                     instance=self.instance)
+            self.stg_ftsk.execute()
+
         return wrap
 
+    def revert_impl(self, result, flow_failures, **kwargs):
+        # Only reverts failed rebuilds, because the revert
+        # for a failed build is handled in the manager.
 
-class Resize(task.Task):
+        if self.instance.task_state == task_states.REBUILD_SPAWNING:
+            LOG.info(_LI('Rebuild of instance %s failed. '
+                     'Deleting instance from destination.'),
+                     self.instance.name, instance=self.instance)
+            vm.dlt_lpar(self.adapter, vm.get_pvm_uuid(self.instance))
+
+
+class Resize(pvm_task.PowerVMTask):
 
     """The task for resizing an existing VM."""
 
@@ -120,53 +154,44 @@ class Resize(task.Task):
         :param name: VM name to use for the update.  Used on resize when we
             want to rename it but not use the instance name.
         """
-        super(Resize, self).__init__(name='resize_lpar',
-                                     provides='lpar_wrap')
+        super(Resize, self).__init__(
+            instance, 'resize_vm', provides='lpar_wrap')
         self.adapter = adapter
         self.host_wrapper = host_wrapper
-        self.instance = instance
         self.flavor = flavor
         self.vm_name = name
 
-    def execute(self):
-        LOG.info(_LI('Resizing instance: %s'), self.instance.name,
-                 instance=self.instance)
-        wrap = vm.update(self.adapter, self.host_wrapper,
+    def execute_impl(self):
+        return vm.update(self.adapter, self.host_wrapper,
                          self.instance, self.flavor, entry=None,
                          name=self.vm_name)
-        return wrap
 
 
-class Rename(task.Task):
+class Rename(pvm_task.PowerVMTask):
 
     """The task for renaming an existing VM."""
 
-    def __init__(self, adapter, host_uuid, instance, name):
+    def __init__(self, adapter, instance, name):
         """Creates the Task to rename a VM.
 
         Provides the 'lpar_wrap' for other tasks.
 
         :param adapter: The adapter for the pypowervm API
-        :param host_uuid: The managed system uuid
         :param instance: The nova instance.
         :param name: The new VM name.
         """
-        super(Rename, self).__init__(name='rename_lpar_%s' % name,
-                                     provides='lpar_wrap')
+        super(Rename, self).__init__(
+            instance, 'rename_vm_%s' % name, provides='lpar_wrap')
         self.adapter = adapter
-        self.host_uuid = host_uuid
-        self.instance = instance
         self.vm_name = name
 
-    def execute(self):
+    def execute_impl(self):
         LOG.info(_LI('Renaming instance to name: %s'), self.name,
                  instance=self.instance)
-        wrap = vm.rename(self.adapter, self.host_uuid, self.instance,
-                         self.vm_name)
-        return wrap
+        return vm.rename(self.adapter, self.instance, self.vm_name)
 
 
-class PowerOn(task.Task):
+class PowerOn(pvm_task.PowerVMTask):
 
     """The task to power on the instance."""
 
@@ -175,7 +200,7 @@ class PowerOn(task.Task):
         """Create the Task for the power on of the LPAR.
 
         Obtains LPAR info through requirement of lpar_wrap (provided by
-        tf_crt_lpar).
+        tf_crt_vm).
 
         :param adapter: The pypowervm adapter.
         :param host_uuid: The host UUID.
@@ -186,21 +211,19 @@ class PowerOn(task.Task):
                             successfully started.  If True, the Task waits for
                             the pypowervm PowerOn Job to complete.
         """
-        super(PowerOn, self).__init__(name='pwr_lpar',
-                                      requires=['lpar_wrap'])
+        super(PowerOn, self).__init__(
+            instance, 'pwr_vm', requires=['lpar_wrap'])
         self.adapter = adapter
         self.host_uuid = host_uuid
-        self.instance = instance
         self.pwr_opts = pwr_opts
         self.synchronous = synchronous
 
-    def execute(self, lpar_wrap):
-        LOG.info(_LI('Powering on instance: %s'), self.instance.name)
+    def execute_impl(self, lpar_wrap):
         power.power_on(lpar_wrap, self.host_uuid, add_parms=self.pwr_opts,
                        synchronous=self.synchronous)
 
-    def revert(self, lpar_wrap, result, flow_failures):
-        LOG.info(_LI('Powering off instance: %s'), self.instance.name)
+    def revert_impl(self, lpar_wrap, result, flow_failures):
+        LOG.warning(_LW('Powering off instance: %s'), self.instance.name)
 
         if isinstance(result, task_fail.Failure):
             # The power on itself failed...can't power off.
@@ -210,7 +233,7 @@ class PowerOn(task.Task):
         power.power_off(lpar_wrap, self.host_uuid, force_immediate=True)
 
 
-class PowerOff(task.Task):
+class PowerOff(pvm_task.PowerVMTask):
 
     """The task to power off a VM."""
 
@@ -224,20 +247,18 @@ class PowerOff(task.Task):
         :param instance: The nova instance.
         :param force_immediate: Boolean. Perform a VSP hard power off.
         """
-        super(PowerOff, self).__init__(name='pwr_off_lpar')
+        super(PowerOff, self).__init__(instance, 'pwr_off_vm')
         self.adapter = adapter
         self.host_uuid = host_uuid
         self.lpar_uuid = lpar_uuid
-        self.instance = instance
         self.force_immediate = force_immediate
 
-    def execute(self):
-        LOG.info(_LI('Powering off instance %s.'), self.instance.name)
+    def execute_impl(self):
         vm.power_off(self.adapter, self.instance, self.host_uuid,
                      force_immediate=self.force_immediate)
 
 
-class StoreNvram(task.Task):
+class StoreNvram(pvm_task.PowerVMTask):
 
     """Store the NVRAM for an instance."""
 
@@ -248,12 +269,11 @@ class StoreNvram(task.Task):
         :param instance: The nova instance.
         :param immediate: boolean whether to update the NVRAM immediately
         """
-        super(StoreNvram, self).__init__(name='store_nvram')
+        super(StoreNvram, self).__init__(instance, 'store_nvram')
         self.nvram_mgr = nvram_mgr
-        self.instance = instance
         self.immediate = immediate
 
-    def execute(self):
+    def execute_impl(self):
         if self.nvram_mgr is None:
             return
 
@@ -267,7 +287,7 @@ class StoreNvram(task.Task):
                           instance=self.instance)
 
 
-class DeleteNvram(task.Task):
+class DeleteNvram(pvm_task.PowerVMTask):
 
     """Delete the NVRAM for an instance from the store."""
 
@@ -277,12 +297,12 @@ class DeleteNvram(task.Task):
         :param nvram_mgr: The NVRAM manager.
         :param instance: The nova instance.
         """
-        super(DeleteNvram, self).__init__(name='delete_nvram')
+        super(DeleteNvram, self).__init__(instance, 'delete_nvram')
         self.nvram_mgr = nvram_mgr
-        self.instance = instance
 
-    def execute(self):
+    def execute_impl(self):
         if self.nvram_mgr is None:
+            LOG.info(_LI("No op for NVRAM delete."), instance=self.instance)
             return
 
         LOG.info(_LI('Deleting NVRAM for instance: %s'),
@@ -297,7 +317,7 @@ class DeleteNvram(task.Task):
                           instance=self.instance)
 
 
-class Delete(task.Task):
+class Delete(pvm_task.PowerVMTask):
 
     """The task to delete the instance from the system."""
 
@@ -308,38 +328,29 @@ class Delete(task.Task):
         :param lpar_uuid: The VM's PowerVM UUID.
         :param instance: The nova instance.
         """
-        super(Delete, self).__init__(name='dlt_lpar')
+        super(Delete, self).__init__(instance, 'dlt_vm')
         self.adapter = adapter
         self.lpar_uuid = lpar_uuid
-        self.instance = instance
 
-    def execute(self):
-        LOG.info(_LI('Deleting instance %s from system.'), self.instance.name,
-                 instance=self.instance)
+    def execute_impl(self):
         vm.dlt_lpar(self.adapter, self.lpar_uuid)
 
 
-class UpdateIBMiSettings(task.Task):
+class UpdateIBMiSettings(pvm_task.PowerVMTask):
 
     """The task to update settings of an ibmi instance."""
 
-    def __init__(self, adapter, instance, host_uuid, boot_type):
+    def __init__(self, adapter, instance, boot_type):
         """Create the Task to update settings of the IBMi VM.
 
         :param adapter: The adapter for the pypowervm API.
         :param instance: The nova instance.
-        :param host_uuid: The host's PowerVM UUID.
         :param boot_type: The boot type of the instance.
         """
         super(UpdateIBMiSettings, self).__init__(
-            name='update_ibmi_settings')
+            instance, 'update_ibmi_settings')
         self.adapter = adapter
-        self.instance = instance
-        self.host_uuid = host_uuid
         self.boot_type = boot_type
 
-    def execute(self):
-        LOG.info(_LI('Update settings of instance %s.'),
-                 self.instance.name)
-        vm.update_ibmi_settings(
-            self.adapter, self.instance, self.host_uuid, self.boot_type)
+    def execute_impl(self):
+        vm.update_ibmi_settings(self.adapter, self.instance, self.boot_type)
